@@ -10,7 +10,7 @@
  *                                                                         *
  *   CHANGE LOG                                                            *
  *   04/04/2005 -- Created by fizzgig (fizzgig@foofus.net)                 *
- *   04/12/2005 -- Final "alpha" implementation                             *
+ *   04/12/2005 -- Final "alpha" implementation                            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -25,7 +25,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  *                                                                         *
  *   This program is released under the GPL with the additional exemption  *
  *   that compiling, linking, and/or using OpenSSL is allowed.             *
@@ -37,10 +37,7 @@
 #include "medusa-net.h"
 #include "uthash.h"
 #include <pthread.h>
-
-#ifdef HAVE_LIBPCRE
-  #include <pcre.h>
-#endif
+#include <regex.h>
 
 #ifdef HAVE_LIBSSL
   #include <openssl/ssl.h>
@@ -318,9 +315,14 @@ int medusaConnectSSLInternal(sConnectParams* pParams, int hSocket)
 
   /* The SSL context can support SSLv2, SSLv3, or both. The default is to use whatever
      the server demands. The module can override this by setting nSSLVersion. */
+
+  /* Debian's OpenSSL has SSLv2 support disabled. */
+#ifndef OPENSSL_NO_SSL2
   if (pParams->nSSLVersion == 2)
     sslContext = SSL_CTX_new(SSLv2_client_method());
-  else if (pParams->nSSLVersion == 3)
+  else
+#endif
+  if (pParams->nSSLVersion == 3)
     sslContext = SSL_CTX_new(SSLv3_client_method());
   else if (pParams->nSSLVersion == (float)3.1)
     sslContext = SSL_CTX_new(TLSv1_client_method());
@@ -470,7 +472,7 @@ char* medusaReceiveDataInternal(int socket, int* nBufferSize, int nConvertNullsT
   {
     writeError(ERR_DEBUG, "Data receive: Data waiting.");
     nBufReceive = medusaReceive(socket, szBufReceive, BUFFER_SIZE);
-    if (nBufReceive < 0)
+    if (nBufReceive <= 0)
     {
       writeError(ERR_DEBUG, "Data receive: Socket indicated data present, but none found.");
       free(szBufReceive);
@@ -489,7 +491,7 @@ char* medusaReceiveDataInternal(int socket, int* nBufferSize, int nConvertNullsT
     free(szBufReceive);
     return NULL;
   }
-  
+
   /* check for any addition data which may have been sent */
   while (medusaDataReadyTimed(socket, nReceiveDelay2sec, nReceiveDelay2usec) > 0)
   {
@@ -544,7 +546,19 @@ int medusaSendInternal(int socket, char *buf, int size, int options)
   {
 #endif
     int nRet;
+    char *bufReceive = NULL;
+    int nReceiveBufferSize = 0;
     
+    /* flush any extraneous data remaining on socket */
+    /*
+    while (medusaDataReadyTimed(socket, 0, 1) > 0) 
+    {
+      bufReceive = medusaReceiveRaw(socket, &nReceiveBufferSize);
+      writeError(ERR_DEBUG, "Purging extraneous data on socket from previous request (size %d): %s", nReceiveBufferSize, bufReceive);
+      if (bufReceive == NULL) { break; }
+    }
+    */
+
     nRet = send(socket, buf, size, options); 
     if (nRet < 0)
     {
@@ -689,9 +703,11 @@ int medusaDataReady(int socket)
 }
 
 /* Check socket status. Return 1 if connection is still valid. */
-int medusaCheckSocket(int socket)
+int medusaCheckSocket(int socket, int usec)
 {
-  if (medusaDataReadyTimed(socket, 0, 1) == 0)
+  writeError(ERR_DEBUG, "wait time: %d", usec);
+  
+  if (medusaDataReadyTimed(socket, 0, usec) == 0)
     return 1;
   else
     return 0;
@@ -702,7 +718,7 @@ int medusaReceive(int socket, char *buf, int length)
   int ret;
 
   ret = medusaReceiveInternal(socket, buf, length);
-  writeError(ERR_DEBUG, "Data received: %s", buf);
+  writeError(ERR_DEBUG, "Data received (%d): %s", ret, buf);
   return ret;
 }
 
@@ -736,34 +752,36 @@ char* medusaReceiveLineDelay(int socket, int* nBufferSize, int nReceiveDelay1, i
   The function will recheck the socket 5 times before giving up finding a match.
   Each recheck uses a larger timeout value.
 */
-#ifdef HAVE_LIBPCRE
 int medusaReceiveRegex(int hSocket, unsigned char **szBufReceive, int* nBufReceive, const char* regex)
 {
   char *szBufReceiveTmp = NULL;
   int nBufReceiveTmp = 0;
-  pcre *re = NULL;
+  regex_t preg;
+  int errcode = REG_NOMATCH;
+  char errmsg[512];
   const char *error = NULL;
   int erroffset = 0;
-  int rc = 0; 
   int nAttempt = 1;
   const unsigned int BUFFER_SIZE = 576;
 
-  writeError(ERR_DEBUG, "PCRE regural expession: \"%s\"", regex);
-  /* PCRE_DOTALL - a dot metacharater in the pattern matches all characters, 
-     including those that indicate newline */
-  re = pcre_compile(regex, PCRE_CASELESS|PCRE_DOTALL, &error, &erroffset, NULL);
-  if (re == NULL)
+  writeError(ERR_DEBUG, "Regural expession: \"%s\"", regex);
+  errcode = regcomp(&preg, regex, REG_EXTENDED|REG_ICASE|REG_NOSUB);
+  if (errcode)
   {
-    writeError(ERR_ERROR, "PCRE compilation failed at offset %d: %s\n", erroffset, error);
+    memset(errmsg, 0, 512);
+    regerror(errcode, &preg, errmsg, 512);
+    writeError(ERR_ERROR, "Regex compilation failed: %s", errmsg);
     return FAILURE;
   }  
 
   *szBufReceive = medusaReceiveDataInternal(hSocket, nBufReceive, 0, READ_WAIT_TIME, 0);
+  if (*szBufReceive == NULL)
+    return FAILURE;
 
   do
   {
-    rc = pcre_exec(re, NULL, *szBufReceive, *nBufReceive, 0, 0, NULL, 0); 
-    if (rc < 0)
+    errcode = regexec(&preg, *szBufReceive, 0, 0, 0);
+    if (errcode == REG_NOMATCH)
     {
       writeError(ERR_DEBUG, "Failed to match regex. Checking for additional data.");
 
@@ -784,7 +802,7 @@ int medusaReceiveRegex(int hSocket, unsigned char **szBufReceive, int* nBufRecei
         if (*nBufReceive + nBufReceiveTmp > BUFFER_SIZE) 
         {
           writeError(ERR_DEBUG, "Additional data received. Increasing receive buffer %d bytes to %d.", nBufReceiveTmp, *nBufReceive + nBufReceiveTmp + 1);
-          *szBufReceive = realloc(szBufReceive, *nBufReceive + nBufReceiveTmp + 1);
+          *szBufReceive = realloc(*szBufReceive, *nBufReceive + nBufReceiveTmp + 1);
         }
 
         memcpy(*szBufReceive + *nBufReceive, szBufReceiveTmp, nBufReceiveTmp);
@@ -802,27 +820,16 @@ int medusaReceiveRegex(int hSocket, unsigned char **szBufReceive, int* nBufRecei
     }
     else
     {
+      regfree(&preg);
       writeError(ERR_DEBUG, "Successfully matched regex.");
       return SUCCESS;
     }
   } while (nAttempt <= 5);
 
+  regfree(&preg);
   writeError(ERR_ERROR, "Failed to match regex pattern within server's response.");
   return FAILURE;
 }
-#else
-int medusaReceiveRegex(int hSocket, unsigned char **szBufReceive, int* nBufReceive, const char* regex)
-{
-  writeError(ERR_ERROR, "Medusa compiled without PCRE support. Data receiving may be less reliable.");
-
-  *szBufReceive = medusaReceiveDataInternal(hSocket, nBufReceive, 0, READ_WAIT_TIME, 0);
-
-  if (*szBufReceive == NULL)
-    return FAILURE;
-  else
-    return SUCCESS;
-}
-#endif
 
 int medusaSend(int socket, char *buf, int size, int options)
 {
