@@ -23,7 +23,7 @@
  *
 */
 
-#define VERSION_SVN "$Id: medusa.c 1271 2009-11-25 00:24:58Z jmk $" 
+#define VERSION_SVN "$Id: medusa.c 1666 2012-02-09 19:40:02Z jmk $" 
 
 #include <dlfcn.h>
 #include "medusa.h"
@@ -75,6 +75,7 @@ void usage()
   writeVerbose(VB_NONE, "  -g [NUM]     : Give up after trying to connect for NUM seconds (default 3)"); 
   writeVerbose(VB_NONE, "  -r [NUM]     : Sleep NUM seconds between retry attempts (default 3)");   
   writeVerbose(VB_NONE, "  -R [NUM]     : Attempt NUM retries before giving up. The total number of attempts will be NUM + 1.");
+  writeVerbose(VB_NONE, "  -c [NUM]     : Time to wait in usec to verify socket is available (default 500 usec).");
   writeVerbose(VB_NONE, "  -t [NUM]     : Total number of logins to be tested concurrently");
   writeVerbose(VB_NONE, "  -T [NUM]     : Total number of hosts to be tested concurrently");
   writeVerbose(VB_NONE, "  -L           : Parallelize logins using one username per thread. The default is to process ");
@@ -112,6 +113,7 @@ int checkOptions(int argc, char **argv, sAudit *_psAudit)
   _psAudit->iTimeout = DEFAULT_WAIT_TIME;             /* Default wait of 3 seconds */
   _psAudit->iRetryWait = WAIT_BETWEEN_CONNECT_RETRY;  /* Default wait of 3 seconds */
   _psAudit->iRetries = MAX_CONNECT_RETRY;             /* Default of 2 retries (3 total attempts) */
+  _psAudit->iSocketWait = 500;                        /* Default wait of 500 usec */
   _psAudit->iShowModuleHelp = 0;
   iVerboseLevel = 5;
   iErrorLevel = 5;
@@ -128,7 +130,7 @@ int checkOptions(int argc, char **argv, sAudit *_psAudit)
   if (nIgnoreBanner == 0)
     writeVerbose(VB_NONE, "%s v%s [%s] (C) %s %s\n", PROGRAM, VERSION, WWW, AUTHOR, EMAIL);
 
-  while ((opt = getopt(argc, argv, "h:H:u:U:p:P:C:O:e:M:m:g:r:R:t:T:n:bqdsLfFVv:w:Z:")) != EOF)
+  while ((opt = getopt(argc, argv, "h:H:u:U:p:P:C:O:e:M:m:g:r:R:c:t:T:n:bqdsLfFVv:w:Z:")) != EOF)
   {
     switch (opt)
     {
@@ -312,6 +314,9 @@ int checkOptions(int argc, char **argv, sAudit *_psAudit)
     case 'R':
       _psAudit->iRetries = atoi(optarg);
       break;
+    case 'c':
+      _psAudit->iSocketWait = atoi(optarg);
+      break;
     case 'Z':
       _psAudit->pOptResume = malloc( strlen(optarg) + 1 );
       memset(_psAudit->pOptResume, 0, strlen(optarg) + 1);
@@ -517,6 +522,11 @@ void loadFile(char *pFile, char **pFileContent, int *iFileCnt)
     *ptr = '\0';  /* extra NULL to identify end of list */
   }
 
+  if((*iFileCnt) == 0)
+  {
+    writeError(ERR_FATAL, "Error loading user supplied file (%s) -- file may be empty.", pFile);
+  }
+
   free(pFile);
   return;
 }
@@ -538,7 +548,8 @@ void loadFile(char *pFile, char **pFileContent, int *iFileCnt)
 
   Medusa also supports using PwDump files as a combo file. The format of these
   files should be user:id:lm:ntlm. We look for ':::' at the end of the first line
-  to determine if the file contains PwDump output.
+  to determine if the file contains PwDump output. In addition, a LM/NTLM hash
+  pair can be supplied in lieu of a password (e.g. host:user:lm:ntlm).
 */
 int processComboFile(sAudit **_psAudit)
 {
@@ -588,7 +599,7 @@ int processComboFile(sAudit **_psAudit)
     }
   }
 
-  if (iColonCount != 2)
+  if ( ! ((iColonCount == 2) || (iColonCount == 3)) )
   {
     writeError(ERR_DEBUG, "[processComboFile] Number of colons detected in first entry: %d", iColonCount);
     writeError(ERR_FATAL, "Invalid combo file format.");
@@ -754,10 +765,10 @@ char* findNextHost(sAudit *_psAudit, char *_pHost)
   }
   else if (_psAudit->HostType == L_FILE)
   {
-    _pHost = _psAudit->pGlobalHost;
-
     if (*_psAudit->pGlobalHost != '\0')
     {
+      _pHost = _psAudit->pGlobalHost;
+
       /* advancing host list */
       while (*_psAudit->pGlobalHost != '\0')
         _psAudit->pGlobalHost++;
@@ -1528,12 +1539,9 @@ void *startLoginThreadPool(void *arg)
   int iLoginId = 0;
   int iLoginCnt = _psServer->psAudit->iLoginCnt;
  
-  struct hostent *psHostent = NULL; 
-  struct hostent sHostent;
-  struct hostent *hp = NULL;
-  int herr, res;
-  char *tmphstbuf = NULL;
-  size_t hstbuflen;
+  struct addrinfo hints, *res;
+  int errcode;
+  void *ptr;
 
   writeError(ERR_DEBUG_SERVER, "Server ID: %d Host: %s iUserPassCnt: %d iLoginCnt: %d", _psServer->iId, _psServer->psHost->pHost, _psServer->psHost->iUserPassCnt, iLoginCnt);
   
@@ -1547,53 +1555,39 @@ void *startLoginThreadPool(void *arg)
   }
   
   /* resolve host name */
-  hstbuflen = 1024;
-  tmphstbuf = (char *)malloc(hstbuflen);
+  _psServer->pHostIP = malloc(100);
+  memset(_psServer->pHostIP, 0, 100);
 
-/* gethostbyname_r does not appear to be present on NetBSD... we need to move away from gethostbyname/gethostbyname_r... */
-#ifdef HAVE_GETHOSTBYNAME_R
-  while ((res = gethostbyname_r(_psServer->psHost->pHost, &sHostent, tmphstbuf, hstbuflen, &hp, &herr)) && (errno == ERANGE))
+  memset(&hints, 0, sizeof (hints));
+  hints.ai_family = PF_UNSPEC;
+  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_flags |= AI_CANONNAME;
+
+  errcode = getaddrinfo(_psServer->psHost->pHost, NULL, &hints, &res);
+  if (errcode != 0)
   {
-    writeError(ERR_DEBUG_SERVER, "Increasing buffer size for host name resolution.");
-    hstbuflen *= 2;
-    tmphstbuf = (char *)realloc(tmphstbuf, hstbuflen);
-  }
-
-  if (res != 0) {
-    FREE(tmphstbuf);
-    tmphstbuf = NULL;
-    writeError(ERR_CRITICAL, "Failed to resolve hostname: %s", _psServer->psHost->pHost);
+    writeError(ERR_CRITICAL, "Failed to resolve hostname: %s - %s", _psServer->psHost->pHost, gai_strerror(errcode));
     return;
   }
-  else
-  {
-    _psServer->pHostIP = inet_ntoa(* (struct in_addr *) sHostent.h_addr);
-    writeError(ERR_DEBUG_SERVER, "Set host IP: %s", _psServer->pHostIP);
 
-    if (inet_addr(_psServer->pHostIP) == INADDR_NONE)
-    {
-      writeError(ERR_CRITICAL, "'%s' is not a valid IP address (inet_addr failed)", _psServer->pHostIP);
-      return;
-    }
-  }
-#else
-  psHostent = gethostbyname(_psServer->psHost->pHost);
-  if (!psHostent) {
-    writeError(ERR_CRITICAL, "Failed to resolve hostname: %s", _psServer->psHost->pHost);
-    return;
-  }
-  else
-  {
-    _psServer->pHostIP = inet_ntoa(* (struct in_addr *) psHostent->h_addr);
-    writeError(ERR_DEBUG_SERVER, "Set host IP: %s", _psServer->pHostIP);
+  if (res->ai_next != NULL)
+    writeError(ERR_ERROR, "Hostname resolved to multiple addresses. Selecting first address for testing.");
 
-    if (inet_addr(_psServer->pHostIP) == INADDR_NONE)
-    {
-      writeError(ERR_CRITICAL, "'%s' is not a valid IP address (inet_addr failed)", _psServer->pHostIP);
-      return;
-    }
+  inet_ntop (res->ai_family, res->ai_addr->sa_data, _psServer->pHostIP, 100);
+
+  switch (res->ai_family)
+  {
+    case AF_INET:
+      ptr = &((struct sockaddr_in *) res->ai_addr)->sin_addr;
+      break;
+    case AF_INET6:
+      ptr = &((struct sockaddr_in6 *) res->ai_addr)->sin6_addr;
+      break;
   }
-#endif
+
+  inet_ntop (res->ai_family, ptr, _psServer->pHostIP, 100);
+  writeError(ERR_DEBUG_SERVER, "Set IPv%d address: %s (%s)",res->ai_family == PF_INET6 ? 6 : 4, _psServer->pHostIP, res->ai_canonname);
+  freeaddrinfo(res);
 
   /* add login tasks to pool queue */
   for (iLoginId = 0; iLoginId < iLoginCnt; iLoginId++)
@@ -1639,7 +1633,7 @@ void *startLoginThreadPool(void *arg)
     problem, we kick off a single thread to run through these.
   */
   iLoginId = 0;
-  while (_psServer->iCredentialsMissed > 0)
+  if ((_psServer->psAudit->iStatus != AUDIT_ABORT) && (_psServer->iCredentialsMissed > 0))
   {
     writeError(ERR_DEBUG_SERVER, "Adding new clean-up login task to server queue (%d) for %d missed logins", _psServer->iId, _psServer->iCredentialsMissed);
    
@@ -1678,7 +1672,9 @@ void *startLoginThreadPool(void *arg)
   }
 
   writeError(ERR_DEBUG_SERVER, "exiting server: %d", _psServer->iId);
-  
+
+  free(_psServer->pHostIP); 
+ 
   return;
 }
 
@@ -1717,6 +1713,9 @@ int startServerThreadPool(sAudit *_psAudit)
   if (_psAudit->iServerCnt > _psAudit->iHostCnt)
     _psAudit->iServerCnt = _psAudit->iHostCnt;
 
+  /* initialize global crypto (OpenSSL, Libgcrypt) variables */
+  init_crypto_locks();
+
   if ((_psAudit->server_pool = thr_pool_create(0, _psAudit->iServerCnt, POOL_THREAD_LINGER, NULL)) == NULL)
   {
     writeError(ERR_ERROR, "Failed to create root server thread pool.");
@@ -1848,6 +1847,8 @@ int startServerThreadPool(sAudit *_psAudit)
     if (pthread_mutex_init(&(psServer[iServerId].ptmMutex), NULL) != 0)
       writeError(ERR_FATAL, "Server (%d) mutex destroy call failed - %s\n", iServerId, strerror( errno ) );
   }
+  
+  kill_crypto_locks();
 
   return SUCCESS;
 } 
