@@ -36,8 +36,8 @@
 #define MODULE_NAME    "ftp.mod"
 #define MODULE_AUTHOR  "pMonkey <pmonkey@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for FTP/FTPS sessions"
-#define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: ftp.c 1296 2010-02-04 19:19:36Z jmk $"
+#define MODULE_VERSION    "2.1"
+#define MODULE_VERSION_SVN "$Id: ftp.c 1665 2012-02-09 19:35:40Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 
 #define BUF_SIZE 300
@@ -102,7 +102,7 @@ void showUsage()
   writeVerbose(VB_NONE, "");
   writeVerbose(VB_NONE, "  EXPLICIT: AUTH TLS Mode as defined in RFC 4217");
   writeVerbose(VB_NONE, "     Explicit FTPS (FTP/SSL) connects to a FTP service in the clear. Prior to");
-  writeVerbose(VB_NONE, "     sending any credentials, however, an \"AUTH SSL\" command is issued and a");
+  writeVerbose(VB_NONE, "     sending any credentials, however, an \"AUTH TLS\" command is issued and a");
   writeVerbose(VB_NONE, "     SSL session is negotiated.");
   writeVerbose(VB_NONE, "");
   writeVerbose(VB_NONE, "  IMPLICIT: FTP over SSL (990/tcp)");
@@ -159,7 +159,9 @@ int go(sLogin* logins, int argc, char *argv[])
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
 
-        if (strcmp(pOpt, "EXPLICIT") == 0)
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method MODE requires value to be set.");
+        else if (strcmp(pOpt, "EXPLICIT") == 0)
           psSessionData->nAuthType = AUTH_EXPLICIT;
         else if (strcmp(pOpt, "IMPLICIT") == 0)
           psSessionData->nAuthType = AUTH_IMPLICIT;
@@ -185,10 +187,8 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
 {
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
-  char* bufReceive;
+  unsigned char* bufReceive;
   int nReceiveBufferSize = 0;
-  int nStatus = FAILURE;
-  int nBannerStatus = 0;
   sCredentialSet *psCredSet = NULL;
   sConnectParams params;
 
@@ -252,64 +252,46 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         }
 
         /* establish initial connection */
+      
+        /*  http://www.faqs.org/rfcs/rfc959.html
+          
+            Thus the format for multi-line replies is that the first line
+            will begin with the exact required reply code, followed
+            immediately by a Hyphen, "-" (also known as Minus), followed by
+            text.  The last line will begin with the same code, followed
+            immediately by Space <SP>, optionally some text, and the Telnet
+            end-of-line code.
+        */
+        writeError(ERR_DEBUG_MODULE, "[%s] Retrieving FTP banner.", MODULE_NAME);
         bufReceive = NULL;
         nReceiveBufferSize = 0;
-        nStatus = FAILURE;
-        nBannerStatus = 0;
-        while (bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize))
+        
+        /* Grab entire banner and verify format */
+        if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^[0-9]{3,3}-.*\r\n[0-9]{3,3} .*\r\n|^[0-9]{3,3} .*\r\n") == FAILURE) || (bufReceive == NULL))
         {
-          /*  http://www.faqs.org/rfcs/rfc959.html
-            
-              Thus the format for multi-line replies is that the first line
-              will begin with the exact required reply code, followed
-              immediately by a Hyphen, "-" (also known as Minus), followed by
-              text.  The last line will begin with the same code, followed
-              immediately by Space <SP>, optionally some text, and the Telnet
-              end-of-line code.
-          */
-
-          if (strstr(bufReceive, "220 ") != NULL)
-            nBannerStatus = 1;
-          
-          if (nBannerStatus > 0 && (strstr(bufReceive, "\r\n") != NULL))
-            nBannerStatus = 2;
-          
-          if (nBannerStatus > 1)
-          {
-            writeError(ERR_DEBUG_MODULE, "[%s] Server sent '220' code.", MODULE_NAME);
-            nStatus = SUCCESS;
-            FREE(bufReceive);
-            
-            // fizzgig: We may need to receive more banner data here
-            //continue;
-            break;
-          }
-
-          if (strstr(bufReceive, "421 ") != NULL)
-          {
-            writeError(ERR_ERROR, "[%s] Server sent 421 response (too many connections).", MODULE_NAME);
-            nStatus = FAILURE;
-            FREE(bufReceive);
-            break;
-          }          
-          
+          writeError(ERR_DEBUG_MODULE, "[%s] failed: Server sent unknown response. Exiting...", MODULE_NAME);
           FREE(bufReceive);
+          return FAILURE;
         }
 
-        if (nStatus == FAILURE)
+        if (strncmp(bufReceive, "220", 3) == 0)
         {
-          writeError(ERR_DEBUG_MODULE, "[%s] Server did not respond with '220' code. Exiting...", MODULE_NAME);
-          psLogin->iResult = LOGIN_RESULT_UNKNOWN;
-          nState = MSTATE_EXITING;
+          writeError(ERR_DEBUG_MODULE, "[%s] Server sent 220 response.", MODULE_NAME);
+          FREE(bufReceive);
+        }
+        else if (strncmp(bufReceive, "421", 3) == 0)
+        {
+          writeError(ERR_ERROR, "[%s] Server sent 421 response (too many connections).", MODULE_NAME);
+          FREE(bufReceive);
           return FAILURE;
-          break;
         }
         else
         {
-          writeError(ERR_DEBUG_MODULE, "Connected");
-          nState = MSTATE_RUNNING;
-        }
-       
+          writeError(ERR_ERROR, "[%s] Server sent unknown response code: %c%c%c", MODULE_NAME, bufReceive[0], bufReceive[1], bufReceive[2]);
+          FREE(bufReceive);
+          return FAILURE;
+        }          
+
         /* Establish Explicit FTPS mode authentication if requested */ 
         if (_psSessionData->nAuthType == AUTH_EXPLICIT)
         {
@@ -320,12 +302,14 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
           }
         }
 
+        nState = MSTATE_RUNNING;
+
         break;
       case MSTATE_RUNNING:
         /* The FTP service may be configured to drop connections after an arbitrary number of failed
            logon attempts. We will reuse the established connection to send authentication attempts 
            until that disconnect happens. At that point the connection should be reestablished. */
-        if ( medusaCheckSocket(hSocket) )
+        if ( medusaCheckSocket(hSocket, psLogin->psServer->psAudit->iSocketWait) )
         {
           nState = tryLogin(hSocket, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
 
@@ -395,7 +379,7 @@ int initAuthSSL(int hSocket, _MODULE_DATA* _psSessionData)
   writeError(ERR_NOTICE, "[%s] Establishing Explicit FTPS (FTP/SSL) session.", MODULE_NAME);
 
   memset(bufSend, 0, BUF_SIZE);
-  sprintf(bufSend, "AUTH SSL\r\n");
+  sprintf(bufSend, "AUTH TLS\r\n");
   if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
@@ -403,14 +387,15 @@ int initAuthSSL(int hSocket, _MODULE_DATA* _psSessionData)
   }
 
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^[0-9]{3,3}-.*\r\n[0-9]{3,3} .*\r\n|^[0-9]{3,3} .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data.", MODULE_NAME);
+    writeError(ERR_DEBUG_MODULE, "[%s] failed: Server sent unknown or no response. Exiting...", MODULE_NAME);
+    FREE(bufReceive);
     return FAILURE;
   }
+  
   /* 234 Proceed with negotiation. */
-  else if (strstr(bufReceive, "234 ") != NULL)
+  if (strncmp(bufReceive, "234 ", 4) == 0)
   {
     FREE(bufReceive);
 
@@ -420,6 +405,11 @@ int initAuthSSL(int hSocket, _MODULE_DATA* _psSessionData)
       return FAILURE;
     }
   }
+  else
+  {
+    writeError(ERR_ERROR, "[%s] Failed to establish SSL connection. Server sent response: %c%c%c", MODULE_NAME, bufReceive[0], bufReceive[1], bufReceive[2]);
+    return FAILURE;
+  }
 
   return SUCCESS;
 }
@@ -441,10 +431,9 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
   }
  
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^[0-9]{3,3}-.*\r\n[0-9]{3,3} .*\r\n|^[0-9]{3,3} .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] failed: medusaReceive returned no data. Server may have dropped connection due to lack of encryption. Enabling the EXPLICIT mode may help.", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] failed: Server sent unknown or no response. Server may have dropped connection due to lack of encryption or due to anti-bruteforce measures. Enabling EXPLICIT mode may help with the former cause and increasing the socket check delay (e.g. -c 1000) may help with the later.", MODULE_NAME);
     return FAILURE;
   }
 
@@ -457,9 +446,19 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
     
     FREE(bufReceive);
     
-    if (initAuthSSL(hSocket, _psSessionData) == FAILURE)
-      return FAILURE;
-  
+    if ( medusaCheckSocket(hSocket, (*psLogin)->psServer->psAudit->iSocketWait) )
+    {
+      writeError(ERR_DEBUG_MODULE, "[%s] Checking socket status: OK", MODULE_NAME);
+      if (initAuthSSL(hSocket, _psSessionData) == FAILURE)
+        return FAILURE;
+    }
+    else
+    {
+      writeError(ERR_DEBUG_MODULE, "[%s] Checking socket status: FAIL - restart connection", MODULE_NAME);
+      _psSessionData->nAuthType = AUTH_EXPLICIT;      
+      return MSTATE_NEW;
+    }
+
     /* re-send username */
     memset(bufSend, 0, sizeof(bufSend));
     sprintf(bufSend, "USER %.250s\r\n", szLogin);
@@ -470,10 +469,9 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
     }
  
     nReceiveBufferSize = 0;
-    bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-    if (bufReceive == NULL)
+    if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^[0-9]{3,3}-.*\r\n[0-9]{3,3} .*\r\n|^[0-9]{3,3} .*\r\n") == FAILURE) || (bufReceive == NULL))
     {
-      writeError(ERR_ERROR, "%s failed: medusaReceive returned no data.", MODULE_NAME);
+      writeError(ERR_ERROR, "[%s] failed: Server sent unknown or no response. Exiting...", MODULE_NAME);
       return FAILURE;
     }
   }  
@@ -513,13 +511,13 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
   }
 
   nReceiveBufferSize = 0;
-  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^[0-9]{3,3}-.*\r\n[0-9]{3,3} .*\r\n|^[0-9]{3,3} .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "%s failed: medusaReceive returned no data.", MODULE_NAME);
     return FAILURE;
   }
-  else if (bufReceive[0] == '2')
+  
+  if (bufReceive[0] == '2')
   {
     writeError(ERR_DEBUG_MODULE, "%s : Login attempt successful.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_SUCCESS;
@@ -529,7 +527,8 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
   {
     writeError(ERR_DEBUG_MODULE, "%s : Login attempt failed.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_FAIL;
-    iRet = MSTATE_RUNNING;
+    /* Restarting session for now as it's currently faster when dealing with anti-bruteforce services */
+    iRet = MSTATE_NEW;
   }
   
   FREE(bufReceive);
