@@ -3,7 +3,7 @@
 #ifndef _HTTP_DIGEST_H_
 #define _HTTP_DIGEST_H_
 
-#include "modsrc/md5.h"
+#include <openssl/md5.h>
 
 #define HASHLEN 16
 typedef char HASH[HASHLEN];
