@@ -41,7 +41,7 @@
 #define MODULE_AUTHOR  "fizzgig <fizzgig@foofus.net>"
 #define	MODULE_SUMMARY_USAGE	"Brute force module for HTTP"
 #define MODULE_VERSION		"2.0"
-#define MODULE_VERSION_SVN "$Id: http.c 1259 2009-11-16 04:19:46Z jmk $"
+#define MODULE_VERSION_SVN "$Id: http.c 1673 2012-02-14 00:03:14Z jmk $"
 #define MODULE_SUMMARY_FORMAT	"%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define OPENSSL_WARNING "No usable OPENSSL. Module disabled."
@@ -113,9 +113,9 @@ void showUsage()
   writeVerbose(VB_NONE, "%s (%s) %s :: %s\n", MODULE_NAME, MODULE_VERSION, MODULE_AUTHOR, MODULE_SUMMARY_USAGE);
   writeVerbose(VB_NONE, "Available module options:");
   writeVerbose(VB_NONE, "  USER-AGENT:? (User-Agent. Default: Teh Forest Lobster.)");
-  writeVerbose(VB_NONE, "  DIR:? (Target directory. Default \"/\"");
+  writeVerbose(VB_NONE, "  DIR:? (Target directory. Default \"/\")");
   writeVerbose(VB_NONE, "  AUTH:? (Authentication Type (BASIC/DIGEST/NTLM). Default: automatic)");
-  writeVerbose(VB_NONE, "  DOMAIN:? [optional] (BASIC/NTLM)");
+  writeVerbose(VB_NONE, "  DOMAIN:? [optional]");
   writeVerbose(VB_NONE, "");
   writeVerbose(VB_NONE, "Usage example: \"-M http -m USER-AGENT:\"g3rg3 gerg\" -m DIR:exchange/\"");
 }
@@ -140,7 +140,7 @@ int go(sLogin* logins, int argc, char *argv[])
   {
     // Parameters are good - make module go now
     writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
- 
+
     for (i=0; i<argc; i++) {
       pOptTmp = malloc( strlen(argv[i]) + 1);
       memset(pOptTmp, 0, strlen(argv[i]) + 1);
@@ -182,7 +182,9 @@ int go(sLogin* logins, int argc, char *argv[])
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
 
-        if (strcmp(pOpt, "BASIC") == 0)
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method AUTH requires value to be set.");
+        else if (strcmp(pOpt, "BASIC") == 0)
           psSessionData->nAuthType = AUTH_BASIC;
         else if (strcmp(pOpt, "DIGEST") == 0)
           psSessionData->nAuthType = AUTH_DIGEST;
@@ -230,7 +232,7 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* _psLogin)
   int nBufLength = 0;
   sCredentialSet *psCredSet = NULL;
   sConnectParams params;
- 
+
   psCredSet = malloc( sizeof(sCredentialSet) );
   memset(psCredSet, 0, sizeof(sCredentialSet));
 
@@ -248,7 +250,7 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* _psLogin)
     writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME, _psLogin->psServer->pHostIP);
     nState = MSTATE_COMPLETE;
   }
- 
+
   memset(&params, 0, sizeof(sConnectParams));
   if (_psLogin->psServer->psAudit->iPortOverride > 0)
     params.nPort = _psLogin->psServer->psAudit->iPortOverride;
@@ -272,7 +274,7 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* _psLogin)
     memset(_psSessionData->szHostHeader, 0, nBufLength + 1);
     sprintf(_psSessionData->szHostHeader, "%s:%d", _psLogin->psServer->psHost->pHost, params.nPort);
   }
-      
+
   if (!_psSessionData->szUserAgent)
   {
     _psSessionData->szUserAgent = malloc(19);
@@ -304,19 +306,17 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* _psLogin)
       /* Get required authorization method */
       if (_psSessionData->nAuthType == AUTH_UNKNOWN)
       {
-        getAuthType(hSocket, _psSessionData, &_psLogin, params.nPort);
-        
-        if (_psSessionData->nAuthType == AUTH_UNKNOWN)
+        if ((getAuthType(hSocket, _psSessionData, &_psLogin, params.nPort) == FAILURE) || (_psSessionData->nAuthType == AUTH_UNKNOWN))
         {
           _psLogin->iResult = LOGIN_RESULT_UNKNOWN;
           return FAILURE;
         }
-        
+
         nState = MSTATE_NEW;
       }
       else
         nState = MSTATE_RUNNING;
-      
+
       break;
     case MSTATE_RUNNING:
       nState = tryLogin(hSocket, _psSessionData, &_psLogin, params.nPort, psCredSet->psUser->pUser, psCredSet->pPass);
@@ -355,6 +355,7 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* _psLogin)
     default:
       writeError(ERR_CRITICAL, "Unknown HTTP module state (%d). Exiting...", nState);
       _psLogin->iResult = LOGIN_RESULT_UNKNOWN;
+      nState = MSTATE_EXITING;
       break;
     }
   }
@@ -374,7 +375,7 @@ int getAuthType(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int n
 
   nSendBufferSize = 5 + strlen(_psSessionData->szDir) + 17 + strlen(_psSessionData->szHostHeader) +
                     14 + strlen(_psSessionData->szUserAgent) + 4; 
- 
+
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
 
@@ -388,23 +389,22 @@ int getAuthType(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int n
     FREE(bufSend);
     return FAILURE;
   }
-  
+
   FREE(bufSend);
 
-  bufReceive = medusaReceiveLine(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "HTTP/1.* .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "No data received", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] Failed: Unexpected or no data received: %s", MODULE_NAME, bufReceive);
     return FAILURE;
   }
-  
+
   writeError(ERR_DEBUG_MODULE, "[%s] Parsing authentication header: %s", MODULE_NAME, bufReceive);
-  if (strcasestr(bufReceive, "WWW-Authenticate: Basic"))
+  if ((strcasestr(bufReceive, "WWW-Authenticate: Basic")) || (strcasestr(bufReceive, "WWW-Authenticate:Basic")))
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Server requested basic authentication.", MODULE_NAME);
     _psSessionData->nAuthType = AUTH_BASIC;
   }
-  else if (strcasestr(bufReceive, "WWW-Authenticate: NTLM"))
+  else if ((strcasestr(bufReceive, "WWW-Authenticate: NTLM")) || (strcasestr(bufReceive, "WWW-Authenticate:NTLM")))
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Server requested integrated windows authentication.", MODULE_NAME);
     _psSessionData->nAuthType = AUTH_NTLM;
@@ -453,22 +453,22 @@ int sendAuthBasic(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
 
   if (_psSessionData->szDomain)
     FREE(szLoginDomain);
-  
+
   nSendBufferSize = 5 + strlen(_psSessionData->szDir) + 17 + strlen(_psSessionData->szHostHeader) +
                     14 + strlen(_psSessionData->szUserAgent) + 23 + strlen(szEncodedAuth) + 4;
- 
+
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
 
   sprintf(bufSend, "GET /%s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\nAuthorization: Basic %s\r\n\r\n", 
           _psSessionData->szDir, _psSessionData->szHostHeader, _psSessionData->szUserAgent, szEncodedAuth);
-  
+
   if (medusaSend(hSocket, bufSend, nSendBufferSize, 0) < 0)
   {
     writeError(ERR_ERROR, "%s failed: medusaSend was not successful", MODULE_NAME);
     nRet = FAILURE;  
   }
-  
+
   FREE(szEncodedAuth);
   FREE(bufSend);
   return nRet;
@@ -495,33 +495,32 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
 
   base64_encode((char *)&sTmpReq, SmbLength(&sTmpReq), szTmpBuf64);
   writeError(ERR_DEBUG_MODULE, "[%s] Sending initial challenge (B64 Encoded): %s", MODULE_NAME, szTmpBuf64);
-  
+
   nSendBufferSize = 5 + strlen(_psSessionData->szDir) + 17 + strlen(_psSessionData->szHostHeader) +
                     14 + strlen(_psSessionData->szUserAgent) + 22 + strlen(szTmpBuf64) + 28;
- 
+
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
 
   sprintf(bufSend, "GET /%s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\nAuthorization: NTLM %s\r\nConnection: keep-alive\r\n\r\n", 
           _psSessionData->szDir, _psSessionData->szHostHeader, _psSessionData->szUserAgent, szTmpBuf64);
-  
+
   if (medusaSend(hSocket, bufSend, nSendBufferSize, 0) < 0)
   {
     writeError(ERR_ERROR, "%s failed: medusaSend was not successful", MODULE_NAME);
     return FAILURE;
   }
-  
+
   FREE(szTmpBuf64);
   FREE(bufSend);
 
   /* --- Retrieve NTLM challenge from server --- */
-  bufReceive = medusaReceiveLine(hSocket, &nReceiveBufferSize);
-  if (bufReceive == NULL)
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "HTTP/1.* .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] No data received", MODULE_NAME);
+    writeError(ERR_ERROR, "[%s] Failed: Unexpected or no data received: %s", MODULE_NAME, bufReceive);
     return FAILURE;
   }
-  
+
   if (bufReceive[0] == '\0')
   {
     writeError(ERR_ERROR, "[%s] Service did not respond to our authentication request.", MODULE_NAME);
@@ -542,7 +541,7 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
 
   writeError(ERR_DEBUG_MODULE, "[%s] NTLM Challenge (B64 Encoded): %s", MODULE_NAME, szTmpBuf64 + 23);
   base64_decode(szTmpBuf64 + 23, (char *)&sTmpChall);
-  
+
   FREE(bufReceive);
 
   /* --- Send Type-3 NTLM reply --- */
@@ -554,16 +553,16 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
 
   base64_encode((char *)&sTmpResp, SmbLength(&sTmpResp), szTmpBuf64);
   writeError(ERR_DEBUG_MODULE, "[%s] NTLM Response (B64 Encoded): %s", MODULE_NAME, szTmpBuf64);
-  
+
   nSendBufferSize = 5 + strlen(_psSessionData->szDir) + 17 + strlen(_psSessionData->szHostHeader) +
                     14 + strlen(_psSessionData->szUserAgent) + 22 + strlen(szTmpBuf64) + 23;
- 
+
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
 
   sprintf(bufSend, "GET /%s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\nAuthorization: NTLM %s\r\nConnection: close\r\n\r\n", 
           _psSessionData->szDir, _psSessionData->szHostHeader, _psSessionData->szUserAgent, szTmpBuf64);
-  
+
   if (medusaSend(hSocket, bufSend, nSendBufferSize, 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
@@ -596,15 +595,30 @@ int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, cha
   char  szNonceCount[9] = "00000001";
   char *szMethod = "GET";
   char *szQop = NULL;
-  char *szURI = _psSessionData->szDir;
+  char *szURI = NULL;
   char *szOpaque = NULL;
   HASHHEX HA1;
   HASHHEX HA2 = "";
   HASHHEX Response;
-    
+
+  /* uri should start with a "/" */
+  if (strncmp(_psSessionData->szDir, "/", 1) == 0) 
+  {
+    szURI = malloc(strlen(_psSessionData->szDir) + 1);
+    memset(szURI, 0, strlen(_psSessionData->szDir) + 1);
+    strncat(szURI, _psSessionData->szDir, strlen(_psSessionData->szDir));
+  }
+  else
+  {
+    szURI = malloc(1 + strlen(_psSessionData->szDir) + 1);
+    memset(szURI, 0, 1 + strlen(_psSessionData->szDir) + 1);
+    strncat(szURI, "/", 1);
+    strncat(szURI, _psSessionData->szDir, strlen(_psSessionData->szDir));
+  }
+
   /* Send initial request */
   writeError(ERR_DEBUG_MODULE, "[%s] Sending initial request for digest authentication.", MODULE_NAME);
-  
+
   nSendBufferSize = 5 + strlen(_psSessionData->szDir) + 17 + strlen(_psSessionData->szHostHeader) +
                     14 + strlen(_psSessionData->szUserAgent) + 28;
 
@@ -613,13 +627,13 @@ int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, cha
 
   sprintf(bufSend, "GET /%s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: keep-alive\r\n\r\n", 
           _psSessionData->szDir, _psSessionData->szHostHeader, _psSessionData->szUserAgent);
-  
+
   if (medusaSend(hSocket, bufSend, nSendBufferSize, 0) < 0)
   {
     writeError(ERR_ERROR, "%s failed: medusaSend was not successful", MODULE_NAME);
     return FAILURE;
   }
-  
+
   FREE(bufSend);
 
   /* Retrieve digest challenge from server */
@@ -629,7 +643,7 @@ int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, cha
     writeError(ERR_ERROR, "[%s] No data received", MODULE_NAME);
     return FAILURE;
   }
-  
+
   if (bufReceive[0] == '\0')
   {
     writeError(ERR_ERROR, "[%s] Failed to locate digest challenge.", MODULE_NAME);
@@ -656,7 +670,7 @@ int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, cha
   szAuthenticate = malloc (szTmp1 - szTmp + 1);
   memset(szAuthenticate, 0, szTmp1 - szTmp + 1);
   strncpy(szAuthenticate, szTmp, szTmp1 - szTmp);
-  
+
   FREE(bufReceive);
 
   writeError(ERR_DEBUG_MODULE, "[%s] Server WWW-Authenticate Digest Response: %s", MODULE_NAME, szAuthenticate);
@@ -727,7 +741,7 @@ int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, cha
     szNonce = malloc(1);
     memset(szNonce, 0, 1);
   }
-  
+
   /* Extract Digest Quality of Protection (QoP) - If Specified */
   szTmp = (char *)strcasestr(szAuthenticate, "qop=\"");
   if (szTmp)
@@ -745,7 +759,7 @@ int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, cha
     writeError(ERR_DEBUG_MODULE, "[%s] Failed to extract server Quality of Protection (QoP) response.", MODULE_NAME);
     szQop = NULL;
   }
-  
+
   /* Extract Digest Opaque Value - If Specified */
   szTmp = (char *)strcasestr(szAuthenticate, "opaque=\"");
   if (szTmp)
@@ -763,13 +777,13 @@ int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, cha
     writeError(ERR_DEBUG_MODULE, "[%s] Failed to extract server Opaque value.", MODULE_NAME);
     szOpaque = NULL;
   }
-  
+
   FREE(szAuthenticate);
- 
+
   /* Send digest response */
   /* Example: Authorization: Digest username="it5k", realm="Inter-Tel 5000 (00103605AB8A)", nonce="94144a2abae7411d0f6af2b533425497", uri="/", 
                              response="b9c3980ae4e7fb69796772a3bacc5c18"\r\n */
- 
+
   writeError(ERR_DEBUG_MODULE, "[%s] szAlg: %s", MODULE_NAME, szAlg);
   writeError(ERR_DEBUG_MODULE, "[%s] szLogin: %s", MODULE_NAME, szLogin);
   writeError(ERR_DEBUG_MODULE, "[%s] szRealm: %s", MODULE_NAME, szRealm);
@@ -785,7 +799,7 @@ int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, cha
   DigestCalcHA1(szAlg, szLogin, szRealm, szPassword, szNonce, szCNonce, HA1);
   DigestCalcResponse(HA1, szNonce, szNonceCount, szCNonce, szQop, szMethod, szURI, HA2, Response);
   writeError(ERR_DEBUG_MODULE, "[%s] Calculated Digest Response: %s", MODULE_NAME, Response);
-  
+
   /*
     BASE:   Digest username=\"%s\", realm=\"%s\", nonce=\"%s\", uri=\"%s\", algorithm=\"%s\", response=\"%s\"
     QOP:    , qop=%s, nc=%s, cnonce=\"%s\"
@@ -813,7 +827,7 @@ int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, cha
 
   szAuthorization = malloc(nSendBufferSize + 1);
   memset(szAuthorization, 0, nSendBufferSize + 1);
- 
+
   if ( (szQop != NULL) && (szOpaque != NULL) )
     sprintf(szAuthorization, "Digest username=\"%s\", realm=\"%s\", nonce=\"%s\", uri=\"%s\", algorithm=%s, response=\"%s\", qop=%s, nc=00000001, cnonce=\"%s\", opaque=\"%s\"",
                              szLogin, szRealm, szNonce, szURI, szAlg, Response, szQop, szCNonce, szOpaque);
@@ -832,6 +846,7 @@ int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, cha
   FREE(szNonce);
   FREE(szQop);
   FREE(szOpaque);
+  FREE(szURI);
 
   nSendBufferSize = 5 + strlen(_psSessionData->szDir) + 17 + strlen(_psSessionData->szHostHeader) +
                     14 + strlen(_psSessionData->szUserAgent) + 17 + strlen(szAuthorization) + 28;
@@ -841,7 +856,7 @@ int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, cha
 
   sprintf(bufSend, "GET /%s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\nAuthorization: %s\r\nConnection: keep-alive\r\n\r\n", 
           _psSessionData->szDir, _psSessionData->szHostHeader, _psSessionData->szUserAgent, szAuthorization);
-  
+
   if (medusaSend(hSocket, bufSend, nSendBufferSize, 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
@@ -854,7 +869,7 @@ int sendAuthDigest(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, cha
 
 int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPort, char* szLogin, char* szPassword)
 {
-  char* pReceiveBuffer = NULL;
+  unsigned char* pReceiveBuffer = NULL;
   int nReceiveBufferSize = 0;
   int nRet = SUCCESS;
   char* pTemp = NULL;
@@ -892,59 +907,63 @@ int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPor
   }
 
   writeError(ERR_DEBUG_MODULE, "[%s] Retrieving server response.", MODULE_NAME);
-  pReceiveBuffer = medusaReceiveLine(hSocket, &nReceiveBufferSize);
-  if (pReceiveBuffer == NULL)
+  nReceiveBufferSize = 0;
+  if ((medusaReceiveRegex(hSocket, &pReceiveBuffer, &nReceiveBufferSize, "HTTP/1.* [0-9]{3,3} .*\r\n") == FAILURE) || (pReceiveBuffer == NULL))
   {
-    writeError(ERR_ERROR, "[%s] No data received", MODULE_NAME);
-    (*login)->iResult = LOGIN_RESULT_UNKNOWN;
-    setPassResult(*login, szPassword);
-    return MSTATE_EXITING;  
-  }
-    
-  while ((strstr(pReceiveBuffer, "HTTP/1.") == NULL) && (pReceiveBuffer[0] != '\0'))
-  {
-    FREE(pReceiveBuffer);
-    pReceiveBuffer = medusaReceiveLine(hSocket, &nReceiveBufferSize);
-  }
-
-  if ((pReceiveBuffer == NULL) || (pReceiveBuffer[0] == '\0'))
-  {
-    writeError(ERR_ERROR, "[%s] No data received", MODULE_NAME);
-    (*login)->iResult = LOGIN_RESULT_UNKNOWN;
-    setPassResult(*login, szPassword);
-    return MSTATE_EXITING;  
+    writeError(ERR_ERROR, "[%s] Failed: Unexpected or no data received: %s", MODULE_NAME, pReceiveBuffer);
+    return FAILURE;
   }
 
   pTemp = strstr(pReceiveBuffer, "HTTP/1.");
   pTemp = ((char*)index(pTemp, ' ')) + 1;
-  if (*pTemp == '2' || strncmp(pTemp, "301", 3) == 0 ) 
-  {
-    (*login)->iResult = LOGIN_RESULT_SUCCESS;
-    setPassResult(*login, szPassword);
-    FREE(pReceiveBuffer);
-    return MSTATE_NEW;
-  } 
-  else 
-  {
-    if (*pTemp != '4')
-    {
-      writeVerbose(VB_GENERAL, "Unusual return code for %s:%s (%.3s)", szLogin, szPassword, pTemp);
-      (*login)->iResult = LOGIN_RESULT_UNKNOWN;
-      setPassResult(*login, szPassword);
-      FREE(pReceiveBuffer);
-      return MSTATE_EXITING;   
-    }
-    
-    if (strncmp(pTemp, "404", 3) == 0)
-      writeVerbose(VB_GENERAL, "Page not found at %s:%d", (*login)->psServer->pHostIP, nPort);
-    
-    writeError(ERR_DEBUG_MODULE, "Login failed: error code was %.3s", pTemp);
-    (*login)->iResult = LOGIN_RESULT_FAIL;
-    setPassResult(*login, szPassword);
-    FREE(pReceiveBuffer);
+  memset(pTemp + 3, 0, 1);
 
-    return MSTATE_NEW;
+  switch (atoi(pTemp))
+  {
+    case 200:
+      writeError(ERR_DEBUG_MODULE, "[%s] 200 OK", MODULE_NAME);
+      (*login)->iResult = LOGIN_RESULT_SUCCESS;
+      nRet = MSTATE_NEW; 
+     break;
+    case 301:
+      writeError(ERR_DEBUG_MODULE, "[%s] 301 Moved Permanently", MODULE_NAME);
+      (*login)->iResult = LOGIN_RESULT_SUCCESS;
+      nRet = MSTATE_NEW; 
+     break;
+    case 302:
+      writeError(ERR_DEBUG_MODULE, "[%s] 302 Found", MODULE_NAME);
+      (*login)->iResult = LOGIN_RESULT_SUCCESS;
+      nRet = MSTATE_NEW; 
+      break;
+    case 401:
+      writeError(ERR_DEBUG_MODULE, "[%s] 401 Unauthorized", MODULE_NAME);
+      (*login)->iResult = LOGIN_RESULT_FAIL;
+      nRet = MSTATE_NEW; 
+      break;
+    case 403:
+      writeError(ERR_DEBUG_MODULE, "[%s] 403 Forbidden", MODULE_NAME);
+      (*login)->iResult = LOGIN_RESULT_SUCCESS;
+      nRet = MSTATE_NEW; 
+      break;
+    case 404:
+      writeError(ERR_DEBUG_MODULE, "[%s] 404 Not Found", MODULE_NAME);
+      (*login)->iResult = LOGIN_RESULT_SUCCESS;
+      nRet = MSTATE_NEW; 
+      break;
+    default:
+      writeError(ERR_ERROR, "Unexpected return code for %s:%s (%.3s)", szLogin, szPassword, pTemp);
+
+      (*login)->pErrorMsg = malloc( 27 + 1 );
+      memset((*login)->pErrorMsg, 0, 27 + 1 );
+      sprintf((*login)->pErrorMsg, "Unexpected return code: %.3s", pTemp);
+      (*login)->iResult = LOGIN_RESULT_ERROR;
+      nRet = MSTATE_EXITING; 
+      break;
   }
+
+  FREE(pReceiveBuffer);
+  setPassResult(*login, szPassword);
+  return nRet;  
 }
 
 #else
