@@ -42,7 +42,7 @@
 #define MODULE_AUTHOR  "pMonkey <pmonkey@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for IMAP sessions"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: imap.c 1259 2009-11-16 04:19:46Z jmk $"
+#define MODULE_VERSION_SVN "$Id: imap.c 1723 2012-05-23 16:59:07Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 
 #define BUF_SIZE 300
@@ -178,7 +178,9 @@ int go(sLogin* logins, int argc, char *argv[])
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
 
-        if (strcmp(pOpt, "LOGIN") == 0)
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method AUTH requires value to be set.");
+        else if (strcmp(pOpt, "LOGIN") == 0)
           psSessionData->nAuthType = AUTH_LOGIN;
         else if (strcmp(pOpt, "PLAIN") == 0)
           psSessionData->nAuthType = AUTH_PLAIN;
@@ -268,6 +270,8 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
         if (hSocket > 0)
           medusaDisconnect(hSocket);
 
+        /* Reset SSL flag if set (e.g., SSL connection had been reset by anti-bruteforce mechanism) */
+        params.nUseSSL = 0;
         if (psLogin->psServer->psHost->iUseSSL > 0)
           hSocket = medusaConnectSSL(&params);
         else
@@ -294,7 +298,7 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* psLogin)
         /* The IMAP service may be configured to drop connections after an arbitrary number of failed
            logon attempts. We will reuse the established connection to send authentication attempts 
            until that disconnect happens. At that point the connection should be reestablished. */
-        if ( medusaCheckSocket(hSocket) )
+        if ( medusaCheckSocket(hSocket, psLogin->psServer->psAudit->iSocketWait) )
         {
           nState = tryLogin(hSocket, _psSessionData, &psLogin, psCredSet->psUser->pUser, psCredSet->pPass);
 
@@ -362,7 +366,7 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   int nSendBufferSize = 0;
 
   /* Retrieve IMAP server banner */
-  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\* OK .*\r\n") == FAILURE) || (bufReceive == NULL))
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^\\* OK .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] Failed to retrieve IMAP server banner. Exiting...", MODULE_NAME);
     return FAILURE; 
@@ -401,7 +405,7 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   FREE(bufSend);
 
   nReceiveBufferSize = 0;
-  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "OK CAPABILITY .*\r\n") == FAILURE) || (bufReceive == NULL))
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "OK .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] Failed: No OK message received for CAPABILITY request.", MODULE_NAME);
     return FAILURE;
@@ -426,7 +430,7 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
     FREE(bufSend);
   
     nReceiveBufferSize = 0;
-    if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, " OK .*\r\n") == FAILURE) || (bufReceive == NULL))
+    if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "OK .*\r\n") == FAILURE) || (bufReceive == NULL))
     {
       writeError(ERR_ERROR, "[%s] Failed: No OK message received for STARTTLS request.", MODULE_NAME);
       return FAILURE;
@@ -459,7 +463,7 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
       FREE(bufSend);
 
       nReceiveBufferSize = 0;
-      if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "OK CAPABILITY .*\r\n") == FAILURE) || (bufReceive == NULL))
+      if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "OK .*\r\n") == FAILURE) || (bufReceive == NULL))
       {
         writeError(ERR_ERROR, "[%s] Failed: No OK message received for CAPABILITY request.", MODULE_NAME);
         return FAILURE;
@@ -553,7 +557,7 @@ int sendAuthPlain(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   FREE(bufSend); 
 
   nReceiveBufferSize = 0;
-  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+ *\r\n") == FAILURE) || (bufReceive == NULL))
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+.*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] IMAP server did not respond with \"+\" to AUTHENTICATE PLAIN request.", MODULE_NAME);
     writeError(ERR_ERROR, "[%s] IMAP server sent the following response: %s", MODULE_NAME, bufReceive);
@@ -750,9 +754,26 @@ int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** psLogin, char*
   nReceiveBufferSize = 0;
   if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, ".*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] Failed: Unexpected or no data received.", MODULE_NAME);
-    (*psLogin)->iResult = LOGIN_RESULT_ERROR;
-    nRet = MSTATE_EXITING;
+    /* The IMAP service may be configured to drop connections after an arbitrary number of failed
+       logon attempts. We will reuse the established connection to send authentication attempts 
+       until that disconnect happens.
+ 
+       A default install of Debian (6.0.3 Squeeze) and the Courier Mail Server (0.65) was found 
+       to drop connections immediately after the 8th failed attempt.
+    */
+    if ( medusaCheckSocket(hSocket, (*psLogin)->psServer->psAudit->iSocketWait) )
+    {
+      writeError(ERR_ERROR, "[%s] Failed: Unexpected or no data received.", MODULE_NAME);
+      (*psLogin)->iResult = LOGIN_RESULT_ERROR;
+      nRet = MSTATE_EXITING;
+      return FAILURE;
+    }
+    else
+    {
+      writeError(ERR_NOTICE, "[%s] Socket is no longer valid. Server likely dropped connection. Establishing new session.", MODULE_NAME);
+      (*psLogin)->iResult = LOGIN_RESULT_FAIL;
+      nRet = MSTATE_NEW;
+    }
   }
   else if (strstr(bufReceive,"OK") != NULL) 
   {
@@ -783,6 +804,13 @@ int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** psLogin, char*
     writeError(ERR_DEBUG_MODULE, "[%s] Login attempt failed.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_FAIL;
     nRet = MSTATE_RUNNING;
+    
+    /* Exchange 2010 can include disconnect in message -- NO AUTHENTICATE failed.[0D][0A]* BYE Connection is closed. */
+    if (strstr(bufReceive,"* BYE ") != NULL)
+    {
+      writeError(ERR_NOTICE, "[%s] Socket is no longer valid. Server likely dropped connection. Establishing new session.", MODULE_NAME);
+      nRet = MSTATE_NEW;
+    }
   }
   else if (strstr(bufReceive,"BAD") != NULL) 
   {
