@@ -47,7 +47,7 @@
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for M$-SQL sessions"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: mssql.c 1225 2009-10-13 18:55:27Z jmk $"
+#define MODULE_VERSION_SVN "$Id: mssql.c 1719 2012-05-23 16:56:04Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define OPENSSL_WARNING "No usable OPENSSL. Module disabled."
@@ -246,10 +246,12 @@ int connectMSSQL(sLogin *_psLogin, _MODULE_DATA *_psSessionData)
   int nReceiveBufferSize = 0;
   char *bufReceive = NULL; 
   int nPort = PORT_MSSQL; 
+  int nPortTmp = 0; 
   int hSocket;
   unsigned char pkt_sqlping[] = { 0x02 };
   int nPingResponseLen = 0;
   int nSQLInstance = 0;
+  int nSQLInstancePort = 0;
   char *szTmp = NULL;
   char *szTmp1 = NULL;
   char *szTmp2 = NULL;
@@ -322,25 +324,47 @@ int connectMSSQL(sLogin *_psLogin, _MODULE_DATA *_psSessionData)
       while (szTmp = strstr(szTmp, "ServerName;"))
       {
         szTmp1 = strstr(szTmp, ";;");
+        
+        if (szTmp1 == NULL)
+        {
+            writeError(ERR_ERROR, "[%s] Possible incomplete capture of service information on host %s. There may be additional SQL services.", MODULE_NAME, _psLogin->psServer->pHostIP);
+            break;
+        }
+
         memset(szTmp1, 0, 1);
         szTmp1 += 2; 
         writeError(ERR_DEBUG_MODULE, "[%s] SQL server (%s) ping response (instance: %d) - %s", MODULE_NAME, _psLogin->psServer->pHostIP, nSQLInstance, szTmp);
 
-        if ((szTmp2 = strstr(szTmp, ";tcp;")) == NULL)
+        /* ServerName;MACHINE_NAME;InstanceName;MICROSOFT##SSEE;IsClustered;No;Version;9.00.4035.00;np; */
+        if (strstr(szTmp, "InstanceName;MICROSOFT##SSEE") != NULL)
         {
-          writeError(ERR_ERROR, "[%s] SQL server instance %d on host %s - Failed to locate service port. Server may be configured to \"hide\" port. Setting default hidden value of 2433.", MODULE_NAME, nSQLInstance, _psLogin->psServer->pHostIP);
-          nPort = 2433;
+          writeError(ERR_ERROR, "[%s] Internal database (SQL Server Embedded Edition) identified (NOT TESTED) - server %s", MODULE_NAME, _psLogin->psServer->psHost->pHost);
+          writeVerbose(VB_NONE_FILE, "[%s] Internal database (SQL Server Embedded Edition) identified (NOT TESTED) - server %s\n", MODULE_NAME, _psLogin->psServer->psHost->pHost);
         }
+        /* ServerName;MACHINE_NAME;InstanceName;DATABASE_NAME;IsClustered;No;Version;9.00.3042.00 */
+        else if ((szTmp2 = strstr(szTmp, ";tcp;")) == NULL)
+        {
+          writeError(ERR_ERROR, "[%s] Internal or hidden database identified (NOT TESTED) - server %s. (Default hidden value is 2433/tcp)", MODULE_NAME, _psLogin->psServer->psHost->pHost);
+          writeVerbose(VB_NONE_FILE, "[%s] Internal or hidden database identified (NOT TESTED) - server %s. (Default hidden value is 2433/tcp)\n", MODULE_NAME, _psLogin->psServer->psHost->pHost);
+        }
+        /* ServerName;MACHINE_NAME;InstanceName;MSSQLSERVER;IsClustered;No;Version;8.00.194;tcp;1433;np; */
         else
         {
           szTmp2 += 5; /* skip ";tcp;" */
           if ( index(szTmp2, 0x3B) ) { memset( index(szTmp2, 0x3B), 0, 1); }  /* ";" */
-          nPort = atoi(szTmp2);
+          nPortTmp = atoi(szTmp2);
   
-          if (nSQLInstance == 1)
+          if (nSQLInstancePort == 0)
+          {
+            nPort = nPortTmp;
+            nSQLInstancePort++;
             writeError(ERR_DEBUG_MODULE, "[%s] Connecting to SQL server %s on port %d/tcp.", MODULE_NAME, _psLogin->psServer->pHostIP, nPort);
+          }
           else
-            writeError(ERR_ERROR, "[%s] Additional SQL server identified (NOT TESTED) - server %s on port %d/tcp", MODULE_NAME, _psLogin->psServer->pHostIP, nPort);
+          {
+            writeError(ERR_ERROR, "[%s] Additional SQL server identified (NOT TESTED) - server %s on port %d/tcp", MODULE_NAME, _psLogin->psServer->psHost->pHost, nPortTmp);
+            writeVerbose(VB_NONE_FILE, "[%s] Additional SQL server identified (NOT TESTED) - server %s on port %d/tcp\n", MODULE_NAME, _psLogin->psServer->psHost->pHost, nPortTmp);
+          }
         }
 
         szTmp = szTmp1;
@@ -374,7 +398,7 @@ int connectMSSQL(sLogin *_psLogin, _MODULE_DATA *_psSessionData)
   hSocket = medusaConnect(&params);
   if (hSocket < 0)
   {
-     writeError(ERR_NOTICE, "[%s] Failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, _psLogin->psServer->pHostIP);
+     writeError(ERR_NOTICE, "[%s] Failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, _psLogin->psServer->psHost->pHost);
      _psLogin->iResult = LOGIN_RESULT_UNKNOWN;
      return FAILURE;
    }
