@@ -46,7 +46,7 @@
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for MySQL sessions"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: mysql.c 1225 2009-10-13 18:55:27Z jmk $"
+#define MODULE_VERSION_SVN "$Id: mysql.c 1578 2011-07-07 19:31:43Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define BUF_SIZE 300
 
@@ -155,7 +155,9 @@ int go(sLogin* logins, int argc, char *argv[])
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
 
-        if (strcmp(pOpt, "PASSWORD") == 0)
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method PASS requires value to be set.");
+        else if (strcmp(pOpt, "PASSWORD") == 0)
           psSessionData->hashFlag = PASSWORD;
         else if (strcmp(pOpt, "HASH") == 0)
           psSessionData->hashFlag = HASH;
@@ -616,13 +618,13 @@ int MySQLSessionInit(int hSocket, unsigned char** szSessionSalt)
 
   if (bufReceive[4] < 10)
   {
-    writeError(ERR_ERROR, "%s: Server responded requestion protocol version (%d). Version 10 support required.", MODULE_NAME, bufReceive[4]);
+    writeError(ERR_ERROR, "%s: Server responded requesting protocol version (%d). Version 10 support required.", MODULE_NAME, bufReceive[4]);
     FREE(bufReceive);
     return FAILURE;
   }
   else if (bufReceive[4] > 10)
   {
-    writeError(ERR_WARNING, "%s: Server responded requestion protocol version (%d). Support for versions >10 is unknown.", MODULE_NAME, bufReceive[4]);
+    writeError(ERR_WARNING, "%s: Server responded requesting protocol version (%d). Support for versions >10 is unknown.", MODULE_NAME, bufReceive[4]);
   }
 
   /* check server version */
@@ -630,7 +632,7 @@ int MySQLSessionInit(int hSocket, unsigned char** szSessionSalt)
 
   if (!(strstr(szServerVersion, "3.") || strstr(szServerVersion, "4.") || strstr(szServerVersion, "5.") ))
   {
-    writeError(ERR_ERROR, "%s: Server responded requestion version (%d). Only versions 3.x 4.x 5.x currently supported.", MODULE_NAME, szServerVersion);
+    writeError(ERR_ERROR, "%s: Server responded requesting version (%d). Only versions 3.x, 4.x, and 5.x are currently supported.", MODULE_NAME, szServerVersion);
     FREE(bufReceive);
     return FAILURE;
   }
@@ -643,7 +645,7 @@ int MySQLSessionInit(int hSocket, unsigned char** szSessionSalt)
 
   if (newerauth)
   {
-    /*  retrieve session salt for newer auth */
+    /* retrieve session salt for newer auth */
     *szSessionSalt = malloc(22);
     memset(*szSessionSalt, 0, 22);
     memcpy(*szSessionSalt, bufReceive + strlen(szServerVersion) + 10, 9);
