@@ -36,7 +36,7 @@
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for POP3 sessions"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: pop3.c 1259 2009-11-16 04:19:46Z jmk $"
+#define MODULE_VERSION_SVN "$Id: pop3.c 1722 2012-05-23 16:57:27Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 
 #define BUF_SIZE 300 
@@ -152,7 +152,9 @@ int go(sLogin* logins, int argc, char *argv[])
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
 
-        if (strcmp(pOpt, "AS400") == 0)
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method MODE requires value to be set.");
+        else if (strcmp(pOpt, "AS400") == 0)
           psSessionData->nMode = MODE_AS400;
         else
           writeError(ERR_WARNING, "Invalid value for method MODE.");
@@ -162,7 +164,9 @@ int go(sLogin* logins, int argc, char *argv[])
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
 
-        if (strcmp(pOpt, "USER") == 0)
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method AUTH requires value to be set.");
+        else if (strcmp(pOpt, "USER") == 0)
           psSessionData->nAuthType = AUTH_USER;
         else if (strcmp(pOpt, "PLAIN") == 0)
           psSessionData->nAuthType = AUTH_PLAIN;
@@ -339,7 +343,7 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         /* The POP3 service may be configured to drop connections after an arbitrary number of failed
            logon attempts. We will reuse the established connection to send authentication attempts 
            until that disconnect happens. At that point the connection should be reestablished. */
-        if ( medusaCheckSocket(hSocket) )
+        if ( medusaCheckSocket(hSocket, psLogin->psServer->psAudit->iSocketWait) )
         {
           nState = tryLogin(hSocket, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
 
@@ -417,7 +421,7 @@ int getAuthType(int hSocket, _MODULE_DATA* _psSessionData)
   }
   FREE(bufSend);
 
-  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+OK .*\r\n\\.*\r\n|-ERR.*\r\n") == FAILURE) || (bufReceive == NULL))
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+OK.*\r\n\\.*\r\n|-ERR.*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] Failed: Server did not respond that it supported any of the authentication types we handle (USER, LOGIN, and NTLM). Use the AUTH module option to force the use of an authentication type: %s", MODULE_NAME, bufReceive);
     return FAILURE;
@@ -450,6 +454,11 @@ int getAuthType(int hSocket, _MODULE_DATA* _psSessionData)
       return FAILURE;
     }
   }
+  else if ((strstr(bufReceive,"-ERR") != NULL))
+  {
+    writeError(ERR_ERROR, "[%s] Server did not understand CAPA request. Defaulting to USER authentication type, use \"-m AUTH\" option to specify alternative method.", MODULE_NAME);
+    _psSessionData->nAuthType = AUTH_USER;
+  }
 
   return SUCCESS;
 }
@@ -813,8 +822,25 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
   nReceiveBufferSize = 0;
   if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "\\+OK.*\r\n|-ERR.*\r\n") == FAILURE) || (bufReceive == NULL))
   {
-    writeError(ERR_ERROR, "[%s] Failed: Unexpected or no data received: %s", MODULE_NAME, bufReceive);
-    return FAILURE;
+    /* The POP3 service may be configured to drop connections after an arbitrary number of failed
+       logon attempts. We will reuse the established connection to send authentication attempts 
+       until that disconnect happens.
+ 
+       A default install of Debian (6.0.3 Squeeze) and the Courier Mail Server (0.65) was found 
+       to drop connections immediately after the 8th failed attempt.
+    */
+    if ( medusaCheckSocket(hSocket, (*psLogin)->psServer->psAudit->iSocketWait) )
+    {
+      writeError(ERR_ERROR, "[%s] Failed: Unexpected or no data received.", MODULE_NAME);
+      (*psLogin)->iResult = LOGIN_RESULT_ERROR;
+      return FAILURE;
+    }
+    else
+    {
+      writeError(ERR_NOTICE, "[%s] Socket is no longer valid. Server likely dropped connection. Establishing new session.", MODULE_NAME);
+      (*psLogin)->iResult = LOGIN_RESULT_FAIL;
+      nRet = MSTATE_NEW;
+    }
   }
   else if (bufReceive[0] == '+')
   {
@@ -866,6 +892,12 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
         (*psLogin)->iResult = LOGIN_RESULT_ERROR;
         nRet = MSTATE_EXITING;
       }
+      else if (strstr(bufReceive, "-ERR Logon attempt invalid"))
+      {
+        writeError(ERR_DEBUG_MODULE, "[%s] Generic POP3 invalid attempt message. AS/400 may not be configured for verbose messages.", MODULE_NAME);
+        (*psLogin)->iResult = LOGIN_RESULT_FAIL;
+        nRet = MSTATE_EXITING;
+      }
       else
       {
         writeError(ERR_ERROR, "[%s] Unknown AS/400 error message: %s", MODULE_NAME, bufReceive);
@@ -877,7 +909,8 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
     {
       writeError(ERR_DEBUG_MODULE, "[%s] Login attempt failed.", MODULE_NAME);
       (*psLogin)->iResult = LOGIN_RESULT_FAIL;
-      nRet = MSTATE_RUNNING;
+      //nRet = MSTATE_RUNNING;
+      nRet = MSTATE_NEW;
     }
   }
  
