@@ -88,22 +88,21 @@
 #include <stdlib.h>
 #include "module.h"
 
-#include "md5.h"
-#include "hmacmd5.h"
-
 #define MODULE_NAME    "smbnt.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for SMB (LM/NTLM/LMv2/NTLMv2) sessions"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: smbnt.c 1228 2009-10-14 21:01:40Z jmk $"
+#define MODULE_VERSION_SVN "$Id: smbnt.c 1682 2012-02-29 19:13:03Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define OPENSSL_WARNING "No usable OPENSSL. Module disabled."
 
 #ifdef HAVE_LIBSSL
 
+#include <openssl/md5.h>
 #include <openssl/md4.h>
 #include <openssl/des.h>
+#include "hmacmd5.h"
 
 #define PORT_SMB 139
 #define PORT_SMBNT 445
@@ -213,7 +212,7 @@ void showUsage()
   writeVerbose(VB_NONE, "    PASSWORD: Use normal password.");
   writeVerbose(VB_NONE, "    HASH:     Use a NTLM hash rather than a password.");
   writeVerbose(VB_NONE, "    MACHINE:  Use the machine's NetBIOS name as the password.");
-  writeVerbose(VB_NONE, "  AUTH:?  (LM, NTLM*, LMv2, NTLMv2)");
+  writeVerbose(VB_NONE, "  AUTH:?  (LM, NTLM, LMv2*, NTLMv2)");
   writeVerbose(VB_NONE, "    Option sets LAN Manager Authentication level.");
   writeVerbose(VB_NONE, "    LM: ");
   writeVerbose(VB_NONE, "    NTLM: ");
@@ -263,7 +262,7 @@ int go(sLogin* logins, int argc, char *argv[])
   {
     writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
  
-    psSessionData->authLevel = AUTH_NTLM;
+    psSessionData->authLevel = AUTH_LMv2;
     psSessionData->accntFlag = LOCAL;
     psSessionData->hashFlag = PASSWORD;
     psSessionData->protoFlag = WIN2000_NATIVEMODE;
@@ -280,8 +279,10 @@ int go(sLogin* logins, int argc, char *argv[])
       {
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
-        
-        if (strcmp(pOpt, "LOCAL") == 0)
+
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method GROUP requires value to be set.");
+        else if (strcmp(pOpt, "LOCAL") == 0)
           psSessionData->accntFlag = LOCAL;
         else if (strcmp(pOpt, "DOMAIN") == 0)
           psSessionData->accntFlag = NTDOMAIN;
@@ -307,7 +308,9 @@ int go(sLogin* logins, int argc, char *argv[])
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
         
-        if (strcmp(pOpt, "PASSWORD") == 0)
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method PASS requires value to be set.");
+        else if (strcmp(pOpt, "PASSWORD") == 0)
           psSessionData->hashFlag = PASSWORD;
         else if (strcmp(pOpt, "HASH") == 0)
           psSessionData->hashFlag = HASH;
@@ -320,7 +323,9 @@ int go(sLogin* logins, int argc, char *argv[])
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
         
-        if (strcmp(pOpt, "LM") == 0)
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method AUTH requires value to be set.");
+        else if (strcmp(pOpt, "LM") == 0)
           psSessionData->authLevel = AUTH_LM;
         else if (strcmp(pOpt, "NTLM") == 0)
           psSessionData->authLevel = AUTH_NTLM;
@@ -633,7 +638,7 @@ int HashLM(_SMBNT_DATA *_psSessionData, unsigned char **lmhash, unsigned char *p
                 ((HexChar >= 0x61) && (HexChar <= 0x66)))) {      /* a - f */
             
             writeError(ERR_ERROR, "Error invalid char (%c) for hash.", HexChar);
-            HexChar = 0x30;
+            return FAILURE;
           }
   
           HexChar -= 0x30;
@@ -746,7 +751,7 @@ int MakeNTLM(_SMBNT_DATA *_psSessionData, unsigned char *ntlmhash, unsigned char
                 ((HexChar >= 0x61) && (HexChar <= 0x66)))) {      /* a - f */
             
             writeError(ERR_ERROR, "Error invalid char (%c) for hash.", HexChar);
-            HexChar = 0x30;
+            return FAILURE;
           }
   
           HexChar -= 0x30;
@@ -1584,15 +1589,20 @@ int tryLogin(int hSocket, sLogin** psLogin, _SMBNT_DATA* _psSessionData, char* s
     0xC000006D,         /* STATUS_LOGON_FAILURE */
     0xC000006E,         /* STATUS_ACCOUNT_RESTRICTION */
     0xC000006F,         /* STATUS_INVALID_LOGON_HOURS */
+    0x00C10002,         /* STATUS_INVALID_LOGON_HOURS (LM Authentication) */
     0xC0000070,         /* STATUS_INVALID_WORKSTATION */
+    0x00C00002,         /* STATUS_INVALID_WORKSTATION (LM Authentication) */
     0xC0000071,         /* STATUS_PASSWORD_EXPIRED */
     0xC0000072,         /* STATUS_ACCOUNT_DISABLED */
     0xC000015B,         /* STATUS_LOGON_TYPE_NOT_GRANTED */
     0xC000018D,         /* STATUS_TRUSTED_RELATIONSHIP_FAILURE */
     0xC0000193,         /* STATUS_ACCOUNT_EXPIRED */
+    0x00BF0002,         /* STATUS_ACCOUNT_EXPIRED_OR_DISABLED (LM Authentication) */
     0xC0000224,         /* STATUS_PASSWORD_MUST_CHANGE */
-    0xC0000234,         /* STATUS_ACCOUNT_LOCKED_OUT */
-    0x00050001          /* AS400_STATUS_LOGON_FAILURE */
+    0x00C20002,         /* STATUS_PASSWORD_MUST_CHANGE (LM Authentication) */
+    0xC0000234,         /* STATUS_ACCOUNT_LOCKED_OUT (No LM Authentication Code) */
+    0x00050001,         /* AS400_STATUS_LOGON_FAILURE */
+    0x00000064          /* The machine you are logging onto is protected by an authentication firewall. */
   };
 
   char *smbErrorMsg[] = {
@@ -1603,15 +1613,20 @@ int tryLogin(int hSocket, sLogin** psLogin, _SMBNT_DATA* _psSessionData, char* s
     "STATUS_LOGON_FAILURE",
     "STATUS_ACCOUNT_RESTRICTION",
     "STATUS_INVALID_LOGON_HOURS",
+    "STATUS_INVALID_LOGON_HOURS (LM)",
     "STATUS_INVALID_WORKSTATION",
+    "STATUS_INVALID_WORKSTATION (LM)",
     "STATUS_PASSWORD_EXPIRED",
     "STATUS_ACCOUNT_DISABLED",
     "STATUS_LOGON_TYPE_NOT_GRANTED",
     "STATUS_TRUSTED_RELATIONSHIP_FAILURE",
     "STATUS_ACCOUNT_EXPIRED",
+    "STATUS_ACCOUNT_EXPIRED_OR_DISABLED (LM)",
     "STATUS_PASSWORD_MUST_CHANGE",
+    "STATUS_PASSWORD_MUST_CHANGE (LM)",
     "STATUS_ACCOUNT_LOCKED_OUT",
-    "AS400_STATUS_LOGON_FAILURE"
+    "AS400_STATUS_LOGON_FAILURE",
+    "AUTHENTICATION_FIREWALL_PROTECTION"
   };
 
   memset(&ErrorCode, 0, 10);
@@ -1651,10 +1666,21 @@ int tryLogin(int hSocket, sLogin** psLogin, _SMBNT_DATA* _psSessionData, char* s
       
       iRet = MSTATE_EXITING;
       break;
+    case 0x00006F:  /* STATUS_INVALID_LOGON_HOURS - Valid password */
+    case 0xC10002:  /* STATUS_INVALID_LOGON_HOURS - Valid password (LM) */
+    case 0x000064:  /* Valid password, "The machine you are logging onto is protected by an 
+                       authentication firewall. The specificed account is not allowed to 
+                       authenticate to the machine." */
+    case 0x000070:  /* STATUS_INVALID_WORKSTATION - Valid password */
+    case 0xC00002:  /* STATUS_INVALID_WORKSTATION - Valid password (LM) */
+    case 0x000072:  /* STATUS_ACCOUNT_DISABLED - Valid password */
+    case 0x000193:  /* STATUS_ACCOUNT_EXPIRED - Valid password */
+    case 0xBF0002:  /* STATUS_ACCOUNT_DISABLED or STATUS_ACCOUNT_EXPIRED - Valid password (LM) */
+    case 0x000224:  /* STATUS_PASSWORD_MUST_CHANGE - Valid password */
+    case 0xC20002:  /* STATUS_PASSWORD_MUST_CHANGE - Valid password (LM) */
     case 0x00006E:  /* Valid password, GPO Disabling Remote Connections Using NULL Passwords */
     case 0x00015B:  /* Valid password, GPO "Deny access to this computer from the network" */
-    case 0x000193:  /* Valid password, account expired */ 
-    case 0x000224:  /* Valid password, password expired and must be changed on next logon */
+    case 0x000071:  /* Valid password, password expired and must be changed on next logon */
       (*psLogin)->iResult = LOGIN_RESULT_SUCCESS;
       sprintf(ErrorCode, "0x%6.6X:", SMBerr);
       (*psLogin)->pErrorMsg = malloc( strlen(ErrorCode) + strlen(pErrorMsg) + 1);
