@@ -39,7 +39,7 @@
 #define MODULE_AUTHOR  "pmonkey@foofus.net"
 #define MODULE_SUMMARY_USAGE  "Brute force module for SMTP Authentication with TLS"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: smtp.c 1259 2009-11-16 04:19:46Z jmk $"
+#define MODULE_VERSION_SVN "$Id: smtp.c 1583 2011-07-11 18:51:19Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 
@@ -117,8 +117,8 @@ void showUsage()
   writeVerbose(VB_NONE, "via the username field, but the format appears to differ by auth type.");
   writeVerbose(VB_NONE, "");
   writeVerbose(VB_NONE, "  Usage example: ");
-  writeVerbose(VB_NONE, "    \"medusa -M pop3 -m AUTH:NTLM -U accounts.txt -p password\"");
-  writeVerbose(VB_NONE, "    \"medusa -M pop3 -m EHLO:world -U accounts.txt -p password\"");
+  writeVerbose(VB_NONE, "    \"medusa -M smtp -m AUTH:NTLM -U accounts.txt -p password\"");
+  writeVerbose(VB_NONE, "    \"medusa -M smtp -m EHLO:world -U accounts.txt -p password\"");
   writeVerbose(VB_NONE, "");
 }
 
@@ -154,7 +154,9 @@ int go(sLogin* logins, int argc, char *argv[])
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
 
-        if (strcmp(pOpt, "PLAIN") == 0)
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method AUTH requires value to be set.");
+        else if (strcmp(pOpt, "PLAIN") == 0)
           psSessionData->nAuthType = AUTH_PLAIN;
         else if (strcmp(pOpt, "LOGIN") == 0)
           psSessionData->nAuthType = AUTH_LOGIN;
@@ -277,7 +279,7 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         nState = MSTATE_RUNNING;
         break;
       case MSTATE_RUNNING:
-        if ( medusaCheckSocket(hSocket) )
+        if ( medusaCheckSocket(hSocket, psLogin->psServer->psAudit->iSocketWait) )
         {
           nState = tryLogin(hSocket, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
 
@@ -348,7 +350,7 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   /* Retrieve SMTP banner */
   writeError(ERR_DEBUG_MODULE, "[%s] Retrieving SMTP banner.", MODULE_NAME);  
   nReceiveBufferSize = 0;
-  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "220 .*\r\n") == FAILURE) || (bufReceive == NULL))
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^220 .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_DEBUG_MODULE, "[%s] failed: Server did not respond with '220'. Exiting...", MODULE_NAME);
     FREE(bufReceive);  
@@ -371,7 +373,7 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   FREE(bufSend); 
  
   nReceiveBufferSize = 0;
-  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "250-.*\r\n") == FAILURE) || (bufReceive == NULL))
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^250 .*\r\n|^250-.*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] failed: Server did not respond with '250'. Exiting...", MODULE_NAME);
     FREE(bufReceive);
@@ -397,7 +399,7 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
     FREE(bufSend);
   
     nReceiveBufferSize = 0;
-    if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "220 .*\r\n") == FAILURE) || (bufReceive == NULL))
+    if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^220 .*\r\n") == FAILURE) || (bufReceive == NULL))
     {
       writeError(ERR_ERROR, "[%s] failed: Server did not respond with '220'. Exiting...", MODULE_NAME);
       FREE(bufReceive);
@@ -430,9 +432,9 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
       FREE(bufSend); 
  
       nReceiveBufferSize = 0;
-      if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "220 .*\r\n") == FAILURE) || (bufReceive == NULL))
+      if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^250 .*\r\n|^250-.*\r\n") == FAILURE) || (bufReceive == NULL))
       {
-        writeError(ERR_ERROR, "[%s] failed: Server did not respond with '220'. Exiting...", MODULE_NAME);
+        writeError(ERR_ERROR, "[%s] failed: Server did not respond with '250'. Exiting...", MODULE_NAME);
         FREE(bufReceive);
         return FAILURE;
       }
@@ -515,7 +517,7 @@ int sendAuthPLAIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
 
   /* Server should respond with a 334 response code */
   nReceiveBufferSize = 0;
-  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "334 .*\r\n") == FAILURE) || (bufReceive == NULL))
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^334 .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] SMTP server did not respond with \"334 \" to AUTH PLAIN request.", MODULE_NAME);
     FREE(bufReceive);
@@ -587,7 +589,7 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
 
   /* Server should respond with a base64-encoded username prompt */
   nReceiveBufferSize = 0;
-  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "334 .*\r\n") == FAILURE) || (bufReceive == NULL))
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^334 .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] SMTP server did not respond with \"334 \" to AUTH LOGIN request.", MODULE_NAME);
     FREE(bufReceive);
@@ -642,7 +644,7 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
 
   /* Server should respond with a base64-encoded password prompt */
   nReceiveBufferSize = 0;
-  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "334 .*\r\n") == FAILURE) || (bufReceive == NULL))
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^334 .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] SMTP server did not respond with \"334 \" to AUTH LOGIN request.", MODULE_NAME);
     FREE(bufReceive);  
@@ -727,7 +729,7 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   /* Server should respond with a Base-64 encoded Type-2 challenge message. The challenge response format is 
      "334", followed by a space, followed by the challenge message. */
   nReceiveBufferSize = 0;
-  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "334 .*\r\n") == FAILURE) || (bufReceive == NULL))
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^334 .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] Server did not send valid Type-2 challenge response.", MODULE_NAME);
     FREE(bufReceive);
@@ -803,7 +805,7 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
   writeError(ERR_DEBUG_MODULE, "[%s] Retrieving server response.", MODULE_NAME);
 
   nReceiveBufferSize = 0;
-  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^\\d+ .*\r\n") == FAILURE) || (bufReceive == NULL))
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^[0-9]{3,3} .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] Unknown SMTP server response: %s", MODULE_NAME, bufReceive);
     (*psLogin)->iResult = LOGIN_RESULT_ERROR;
