@@ -37,8 +37,8 @@
 #define MODULE_NAME    "snmp.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for SNMP Community Strings"
-#define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: snmp.c 1225 2009-10-13 18:55:27Z jmk $"
+#define MODULE_VERSION    "2.1"
+#define MODULE_VERSION_SVN "$Id: snmp.c 1399 2010-08-04 21:04:25Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 
@@ -193,7 +193,9 @@ int go(sLogin* logins, int argc, char *argv[])
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
 
-        if ( strcmp(pOpt, "1") == 0 )
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method VERSION requires value to be set.");
+        else if ( strcmp(pOpt, "1") == 0 )
           psSessionData->nVersion = SNMP_VER_V1;
         else if ( strcmp(pOpt, "2C") == 0 )
           psSessionData->nVersion = SNMP_VER_V2C;
@@ -205,7 +207,9 @@ int go(sLogin* logins, int argc, char *argv[])
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
 
-        if ( strcmp(pOpt, "READ") == 0 )
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method ACCESS requires value to be set.");
+        else if ( strcmp(pOpt, "READ") == 0 )
           psSessionData->nReadWrite = SNMP_READ;
         else if ( strcmp(pOpt, "WRITE") == 0 )
           psSessionData->nReadWrite = SNMP_WRITE;
@@ -403,16 +407,84 @@ int initModule(sLogin* psLogin, _SNMP_DATA *_psSessionData)
 
 /* Module Specific Functions */
 
+/*
+  http://book.opensourceproject.org.cn/embedded/tcpipembedded/opensource/0061.html
+
+  The first TLV in the request is called sequence and is used to identify the length of the following TLVs. 
+  The sequence type is 0x30 and the next octet is used to decode the length of this TLV. If the length octet 
+  has the high-order bit set (0x80), then the length octet masked with 0x7f yields the number of bytes that 
+  follow that will make up the value length (in big-endian order). If the high-order bit is not set, then 
+  this octet is the length (no length octets follow).
+  
+  Example:  0x30 0x2f 0x02 0x01 0x00
+  Sequence: 0x30 0x2f
+  Version:  0x02 0x01 0x00
+
+  Example:  0x30 0x82 0x00 0x2f 0x02 0x01 0x00
+  Sequence: 0x30 0x82 0x00 0x2f
+  Version:  0x02 0x01 0x00
+*/
+int parseLength(int nReceiveBufferSize, unsigned char* bufReceive)
+{
+  int nLength = 0;
+  int nOctets = 0;
+
+  if (bufReceive[0] == 0x30)
+  {
+    if (bufReceive[1] & 0x80) /* multi-octet mode */
+    {
+      nOctets = bufReceive[1] & 0x7f;
+
+      /* limited to 4 octets worth of length data */
+      if (nOctets == 1)
+        nLength = bufReceive[2];
+      else if (nOctets == 2)
+        nLength = (bufReceive[2] << 8) + bufReceive[3];
+      else if (nOctets == 3)
+        nLength = (bufReceive[2] << 16) + (bufReceive[3] << 8) + bufReceive[4];
+      else if (nOctets == 4)
+        nLength = (bufReceive[2] << 24) + (bufReceive[3] << 16) + (bufReceive[4] << 8) + bufReceive[5];
+
+      if ((nLength > 0) && ((bufReceive[2+nOctets] == 0x02) && (bufReceive[2+nOctets+1] == 0x01)))
+        writeError(ERR_DEBUG_MODULE, "[%s] Multi-octet mode length: %d", MODULE_NAME, nLength);
+      else
+      {
+        writeError(ERR_ERROR, "[%s] Failed to parse length or SNMP version (multi-octet mode).", MODULE_NAME);
+        nLength = -1;
+      }
+    }
+    else if ((bufReceive[2] == 0x02) && (bufReceive[3] == 0x01))  /* single octet mode, version check */
+    {
+      nLength = bufReceive[1];
+      writeError(ERR_DEBUG_MODULE, "[%s] Single octet mode length: %d", MODULE_NAME, nLength);
+    }
+    else
+    {
+      writeError(ERR_ERROR, "[%s] Failed to parse length or SNMP version.", MODULE_NAME);
+      nLength = -1;
+    }
+  }  
+
+  return nLength;
+}
+
+
 int countResponses(int nReceiveBufferSize, unsigned char* bufReceive)
 {
-  int i, nResponseCount = 0;
-   
+  int i = 0;
+  int nLength = 0;
+  int nResponseCount = 0;
+
   for (i = 0; i < nReceiveBufferSize; i++) {
-    if ((bufReceive[i] == 0x30) && (bufReceive[i+2] == 0x02) && (bufReceive[i+3] == 0x01))
+    if (bufReceive[i] == 0x30)
     {
-      writeError(ERR_DEBUG_MODULE, "[%s] Located start of SNMP response (%d bytes).", MODULE_NAME, bufReceive[i+1]);
-      nResponseCount++;
-      i += bufReceive[i+1];
+      nLength = parseLength(nReceiveBufferSize - i, &bufReceive[i]);
+      if (nLength > 0)
+      {
+        writeError(ERR_DEBUG_MODULE, "[%s] Located start of SNMP response (%d bytes).", MODULE_NAME, nLength);
+        nResponseCount++;
+        i += nLength;
+      }
     }
   }
 
@@ -425,9 +497,9 @@ int processResponse(int nReceiveBufferSize, unsigned char* bufReceive, int *nSNM
   writeError(ERR_DEBUG_MODULE, "[%s] Parsing SNMP response data.", MODULE_NAME);
  
   for (i = 0; i < nReceiveBufferSize; i++) {
-    if ((bufReceive[i] == 0x30) && (bufReceive[i+2] == 0x02) && (bufReceive[i+3] == 0x01))
+    *nSNMPLength = parseLength(nReceiveBufferSize - i, &bufReceive[i]);
+    if (*nSNMPLength > 0)
     {
-      *nSNMPLength = (int)bufReceive[i+1];
       writeError(ERR_DEBUG_MODULE, "[%s] Located start of SNMP response (%d bytes).", MODULE_NAME, *nSNMPLength);
 
       for (; i < nReceiveBufferSize; i++) {
