@@ -36,7 +36,7 @@
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for SSH v2 sessions"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: ssh.c 1295 2010-02-04 19:15:40Z jmk $"
+#define MODULE_VERSION_SVN "$Id: ssh.c 1681 2012-02-29 19:12:47Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define LIBSSH2_WARNING "No usable LIBSSH2. Module disabled."
@@ -53,6 +53,8 @@
 #define SSH_CONN_UNKNOWN 1
 #define SSH_CONN_ESTABLISHED 2
 
+pthread_mutex_t ptmLibSSH2Mutex;
+
 typedef struct __SSH2_DATA {
   char *szBannerMsg;
   int iConnectionStatus;
@@ -120,17 +122,6 @@ int go(sLogin* logins, int argc, char *argv[])
   psSessionData = malloc(sizeof(_SSH2_DATA));
   memset(psSessionData, 0, sizeof(_SSH2_DATA));
 
-#ifdef LIBSSH2_GCRYPT
-  writeVerbose(VB_NONE, "");
-  writeVerbose(VB_NONE, "The default build of Libssh2 is to use OpenSSL for crypto. Several Linux");
-  writeVerbose(VB_NONE, "distributions (e.g. Debian, Ubuntu) build it to use Libgcrypt. Unfortunately,");
-  writeVerbose(VB_NONE, "the implementation within Libssh2 of libgcrypt appears to be broken and is");
-  writeVerbose(VB_NONE, "not thread safe. If you run multiple concurrent Medusa SSH connections, you");
-  writeVerbose(VB_NONE, "are likely to experience segmentation faults. Please help Libssh2 fix this");
-  writeVerbose(VB_NONE, "issue or encourage your distro to use the default Libssh2 build options.");
-  writeVerbose(VB_NONE, "");
-#endif
-
   if ( !(0 <= argc <= 1) )
   {
     // Show usage information
@@ -176,25 +167,6 @@ int go(sLogin* logins, int argc, char *argv[])
   return SUCCESS;
 }
 
-// void *name(size_t count, void **abstract)
-static LIBSSH2_ALLOC_FUNC(ssh2_init_alloc_cb)
-{
-  return malloc(count);
-}
-
-// void name(void *ptr, void **abstract)
-static LIBSSH2_FREE_FUNC(ssh2_init_free_cb)
-{
-  /* Starting with libssh2 0.18 I'm getting a double free with this... */
-  //free(ptr);
-}
-
-// void *name(void *ptr, size_t count, void **abstract)
-static LIBSSH2_REALLOC_FUNC(ssh2_init_realloc_cb)
-{
-  return realloc(ptr, count);
-}
-
 int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
 {
   int hSocket = -1;
@@ -239,6 +211,24 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
     writeError(ERR_DEBUG_MODULE, "[%s] module started for host: %s - no more available users to test.", MODULE_NAME, psLogin->psServer->pHostIP);
     nState = MSTATE_COMPLETE;
   }
+  
+  /* libssh2_init uses a global state, and is not thread safe */      
+  pthread_mutex_lock(&ptmLibSSH2Mutex);
+  
+  // Utilize libssh2_init() once more distro include libssh2 1.2.5 or greater
+  //if (libssh2_init(0))
+  //{
+  //  writeError(ERR_ERROR, "%s: Failed initiating SSH library: Host: %s User: %s Pass: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser, psCredSet->pPass);
+  //  psLogin->iResult = LOGIN_RESULT_UNKNOWN;
+  //  return FAILURE;
+  //}
+
+  // Until then, use:
+  #include <openssl/evp.h>
+  ERR_load_crypto_strings();
+  OpenSSL_add_all_algorithms();
+
+  pthread_mutex_unlock(&ptmLibSSH2Mutex);
 
   while (nState != MSTATE_COMPLETE)
   {  
@@ -252,10 +242,12 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
         if (session)
         {
           writeError(ERR_DEBUG_MODULE, "%s: Destroying previous SSH session structure: Host: %s User: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser);
+          libssh2_session_disconnect(session, "");
           libssh2_session_free(session);
+          session = NULL;
         }
         
-        session = libssh2_session_init_ex(ssh2_init_alloc_cb, ssh2_init_free_cb, ssh2_init_realloc_cb, &ssh2_session_data);
+        session = libssh2_session_init_ex(NULL, NULL, NULL, &ssh2_session_data);
         if (!session)
         {
           writeError(ERR_ERROR, "%s: Failed initiating SSH session: Host: %s User: %s Pass: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser, psCredSet->pPass);
@@ -290,8 +282,14 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
           hSocket = medusaConnect(&params);
           if ( hSocket < 0 ) {
             writeError(ERR_NOTICE, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, psLogin->psServer->pHostIP);
+            
             if (session)
+            {
+              libssh2_session_disconnect(session, "");
               libssh2_session_free(session);
+              session = NULL;
+            }
+            
             psLogin->iResult = LOGIN_RESULT_UNKNOWN;
             return FAILURE;
           }
@@ -315,9 +313,13 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
               else
                 writeError(ERR_ERROR, "%s: Failed establishing SSH session. The following credentials were NOT tested: Host: %s User: %s Pass: %s", MODULE_NAME, psLogin->psServer->pHostIP, psCredSet->psUser->pUser, psCredSet->pPass);
 
-              libssh2_session_disconnect(session, "");
               if (session)
+              {
+                libssh2_session_disconnect(session, "");
                 libssh2_session_free(session);
+                session = NULL;
+              }              
+
               if (hSocket > 0)
                 medusaDisconnect(hSocket);
               hSocket = -1;
@@ -368,6 +370,7 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
         {
           libssh2_session_disconnect(session, "");
           libssh2_session_free(session);
+          session = NULL;
         }
 
         if (hSocket > 0)
@@ -384,6 +387,7 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
         {
           libssh2_session_disconnect(session, "");
           libssh2_session_free(session);
+          session = NULL;
         }
 
         if (hSocket > 0)
@@ -403,32 +407,32 @@ int initModule(sLogin* psLogin, _SSH2_DATA *_psSessionData)
 
 void response_callback(const char* name, int name_len, const char* instruction, int instruction_len, int num_prompts, const LIBSSH2_USERAUTH_KBDINT_PROMPT* prompts, LIBSSH2_USERAUTH_KBDINT_RESPONSE* responses, void **abstract)
 {
-    int i;
-    char *pPass = ((_ssh2_session_data*)(*abstract))->pPass;
-  
-    if (((_ssh2_session_data*)(*abstract))->iAnswerCount > 0)
+  int i;
+  char *pPass = ((_ssh2_session_data*)(*abstract))->pPass;
+
+  if (((_ssh2_session_data*)(*abstract))->iAnswerCount > 0)
+  {
+    writeError(ERR_DEBUG_MODULE, "libssh2 response_callback: sshd asked a question, but we've already given out answer.");
+  }
+  else {
+    for (i=0; i<num_prompts; i++)
     {
-      writeError(ERR_DEBUG_MODULE, "libssh2 response_callback: sshd asked a question, but we've already given out answer.");
-    }
-    else {
-      for (i=0; i<num_prompts; i++)
+      writeError(ERR_DEBUG_MODULE, "libssh2 response_callback: prompt[%d/%d]: %s %d", i + 1, num_prompts, prompts[i].text, prompts[i].length);
+  
+      if ( strcasestr(prompts[i].text, "Password:") != NULL ) {
+        responses[i].text = malloc( strlen(pPass) );
+        memset(responses[i].text, 0, strlen(pPass));
+        strncpy(responses[i].text, pPass, strlen(pPass));
+        responses[i].length = strlen(pPass);
+        writeError(ERR_DEBUG_MODULE, "libssh2 response_callback set password response: %s", pPass);
+        ((_ssh2_session_data*)(*abstract))->iAnswerCount++;
+      }
+      else
       {
-        writeError(ERR_DEBUG_MODULE, "libssh2 response_callback: prompt[%d/%d]: %s %d", i + 1, num_prompts, prompts[i].text, prompts[i].length);
-    
-        if ( strncmp(prompts[i].text, "Password: ", 9) == 0 ) {
-          responses[i].text = malloc( strlen(pPass) );
-          memset(responses[i].text, 0, strlen(pPass));
-          strncpy(responses[i].text, pPass, strlen(pPass));
-          responses[i].length = strlen(pPass);
-          writeError(ERR_DEBUG_MODULE, "libssh2 response_callback set password response: %s", pPass);
-          ((_ssh2_session_data*)(*abstract))->iAnswerCount++;
-        }
-        else
-        {
-          writeError(ERR_ERROR, "%s received an unknown SSH prompt: %s", MODULE_NAME, prompts[i].text);
-        }
+        writeError(ERR_ERROR, "%s received an unknown SSH prompt: %s", MODULE_NAME, prompts[i].text);
       }
     }
+  }
 }
 
 int tryLogin(_SSH2_DATA* _psSessionData, int hSocket, LIBSSH2_SESSION *session, sLogin** psLogin, char* szLogin, char* szPassword)
@@ -449,6 +453,9 @@ int tryLogin(_SSH2_DATA* _psSessionData, int hSocket, LIBSSH2_SESSION *session,
 
   /* libssh2 supports: none, password, publickey, hostbased, keyboard-interactive */
   iAuthMode = SSH_AUTH_UNDEFINED;
+  /*  libssh2_userauth_list returns session->userauth_list_data 
+      libssh2_session_free() call will handle releasing all session data, 
+      including userauth_list_data */
   pErrorMsg = libssh2_userauth_list(session, szLogin, strlen(szLogin));
   if (pErrorMsg)
   {
@@ -457,13 +464,13 @@ int tryLogin(_SSH2_DATA* _psSessionData, int hSocket, LIBSSH2_SESSION *session,
 
     while (pAuth) {
       if (strcmp(pAuth, "password") == 0) {
-        writeError(ERR_DEBUG_MODULE, "Server support user-auth type: password");
+        writeError(ERR_DEBUG_MODULE, "Server supports user-auth type: password");
         iAuthMode = SSH_AUTH_PASSWORD;
         _psSessionData->iConnectionStatus = SSH_CONN_ESTABLISHED;
         break;
       }
       else if (strcmp(pAuth, "keyboard-interactive") == 0) {
-        writeError(ERR_DEBUG_MODULE, "Server support user-auth type: keyboard-interactive");
+        writeError(ERR_DEBUG_MODULE, "Server supports user-auth type: keyboard-interactive");
         iAuthMode = SSH_AUTH_KBDINT;
         _psSessionData->iConnectionStatus = SSH_CONN_ESTABLISHED;
         break;
@@ -471,8 +478,6 @@ int tryLogin(_SSH2_DATA* _psSessionData, int hSocket, LIBSSH2_SESSION *session,
 
       pAuth = strtok_r(NULL, ",", &strtok_ptr);
     }
-  
-    free(pErrorMsg); 
   }
   else if (_psSessionData->iConnectionStatus == SSH_CONN_ESTABLISHED)
   {
