@@ -37,16 +37,16 @@
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for Subversion sessions"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: svn.c 1225 2009-10-13 18:55:27Z jmk $"
+#define MODULE_VERSION_SVN "$Id: svn.c 1632 2011-12-01 22:08:14Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define LIBSVN_WARNING "No usable LIBSVN. Module disabled."
 
 #ifdef HAVE_LIBSVN_CLIENT_1
 
-#include "svn_client.h"
-#include "svn_pools.h"
-#include "svn_config.h"
+#include "subversion-1/svn_client.h"
+#include "subversion-1/svn_pools.h"
+#include "subversion-1/svn_config.h"
 
 #define PORT_SVN 3690
 
@@ -191,14 +191,17 @@ int initModule(sLogin* psLogin, _SVN_DATA *_psSessionData)
   /* set URL - branch, if not specified by user */
   if (_psSessionData->szBranch == NULL)
   {
-    _psSessionData->szBranch = malloc(6);
-    memset(_psSessionData->szBranch, 0, 6);
-    sprintf(_psSessionData->szBranch, "trunk");
+    //_psSessionData->szBranch = malloc(6);
+    //memset(_psSessionData->szBranch, 0, 6);
+    //sprintf(_psSessionData->szBranch, "trunk");
+    _psSessionData->szBranch = malloc(2);
+    memset(_psSessionData->szBranch, 0, 2);
+    sprintf(_psSessionData->szBranch, "/");
   }
 
-  _psSessionData->szURL = malloc(strlen(psLogin->psServer->pHostIP) + strlen(_psSessionData->szBranch) + 10);
-  memset(_psSessionData->szURL, 0, strlen(psLogin->psServer->pHostIP) + strlen(_psSessionData->szBranch) + 10);
-  sprintf(_psSessionData->szURL, "svn://%s/%s", psLogin->psServer->pHostIP, _psSessionData->szBranch);
+  _psSessionData->szURL = malloc(strlen(psLogin->psServer->pHostIP) + log(params.nPort) + strlen(_psSessionData->szBranch) + 10);
+  memset(_psSessionData->szURL, 0, strlen(psLogin->psServer->pHostIP) + log(params.nPort) + strlen(_psSessionData->szBranch) + 10);
+  sprintf(_psSessionData->szURL, "svn://%s:%d/%s", psLogin->psServer->pHostIP, params.nPort, _psSessionData->szBranch);
 
   writeError(ERR_DEBUG_MODULE, "[%s] Set URL: %s", MODULE_NAME, _psSessionData->szURL);
 
@@ -312,10 +315,13 @@ int tryLogin(int hSocket, sLogin** psLogin, _SVN_DATA* _psSessionData, char* szL
   apr_array_header_t *providers = apr_array_make (pool, 1, sizeof (svn_auth_provider_object_t *));
 
   /* Set callback to not retry authentication */
-  svn_client_get_simple_prompt_provider (&provider, svn_prompt_callback, NULL, 0, pool);
-  
+  //svn_client_get_simple_prompt_provider(&provider, svn_prompt_callback, NULL, 0, pool); 
+  svn_auth_get_simple_prompt_provider(&provider, svn_prompt_callback, NULL, 0, pool);
   APR_ARRAY_PUSH (providers, svn_auth_provider_object_t *) = provider;
 
+  //svn_auth_get_username_prompt_provider(&provider, svn_ame_prompt_callback, NULL, 0, pool);
+  //APR_ARRAY_PUSH (providers, svn_auth_provider_object_t *) = provider;
+
   /* Register the auth-providers into the context's auth_baton. */
   svn_auth_open (&ctx->auth_baton, providers, pool);
 
@@ -328,6 +334,7 @@ int tryLogin(int hSocket, sLogin** psLogin, _SVN_DATA* _psSessionData, char* szL
 
   /* Main call into libsvn_client does all the work. */
   err = svn_client_ls (&dirents, _psSessionData->szURL, &revision, FALSE, ctx, pool);
+  //err = svn_client_list2(_psSessionData->szURL, &revision, &revision, svn_depth_empty, SVN_DIRENT_ALL, FALSE, NULL, &ctx->auth_baton, ctx, pool);
   if ((err !=NULL) && err->apr_err == 170001)
   {
     if (strstr(err->message, "Username not found"))
