@@ -48,7 +48,7 @@
 #define MODULE_AUTHOR  "fizzgig <fizzgig@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for telnet sessions"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: telnet.c 1271 2009-11-25 00:24:58Z jmk $"
+#define MODULE_VERSION_SVN "$Id: telnet.c 1580 2011-07-07 21:31:21Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 
 #define PORT_TELNET 23
@@ -163,7 +163,9 @@ int go(sLogin* logins, int argc, char *argv[])
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
 
-        if (strcmp(pOpt, "AS400") == 0)
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method MODE requires value to be set.");
+        else if (strcmp(pOpt, "AS400") == 0)
           psSessionData->nMode = MODE_AS400;
         else
           writeError(ERR_WARNING, "Invalid value for method MODE.");
@@ -362,8 +364,8 @@ int initModule(sLogin* _psLogin, _MODULE_DATA *_psSessionData)
         nState = tryLoginAS400(hSocket, &_psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
       else
         nState = tryLogin(hSocket, &_psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass, nFoundPrompt);
-      
-      if (nState != MSTATE_EXITING)
+
+      if (_psLogin->iResult != LOGIN_RESULT_UNKNOWN) 
       {
         if (processIAClogout(hSocket, _psSessionData) == FAILURE)
         {
@@ -377,7 +379,7 @@ int initModule(sLogin* _psLogin, _MODULE_DATA *_psSessionData)
           the connection. They also seem to ignore "IAC DO LOGOUT" commands. Adding a 
           sleep() hack here, to give them some time to clean-up. 
         */
-        sleep(1);
+        sleep(3);
 
         if (getNextCredSet(_psLogin, psCredSet) == FAILURE)
         {
