@@ -34,8 +34,8 @@
 #define MODULE_NAME    "web-form.mod"
 #define MODULE_AUTHOR  "Luciano Bello <luciano@linux.org.ar>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for web forms"
-#define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: web-form.c 1294 2010-02-04 19:03:49Z jmk $"
+#define MODULE_VERSION    "2.1"
+#define MODULE_VERSION_SVN "$Id: web-form.c 1634 2011-12-02 19:55:09Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define OPENSSL_WARNING "No usable OPENSSL. Module disabled."
@@ -59,6 +59,8 @@ typedef struct __MODULE_DATA {
   char *szFormRest;
   char *szFormUser;
   char *szFormPass;
+  char *szCustomHeader;
+  int nCustomHeaders;
 } _MODULE_DATA;
 
 // Tells us whether we are to continue processing or not
@@ -109,6 +111,8 @@ void showUsage()
   writeVerbose(VB_NONE, "  FORM:?             Target form to request. Default: \"/\"");
   writeVerbose(VB_NONE, "  DENY-SIGNAL:?      Authentication failure message. Attempt flagged as successful if text is not present in");
   writeVerbose(VB_NONE, "                     server response. Default: \"Login incorrect\"");
+  writeVerbose(VB_NONE, "  CUSTOM-HEADER:?    Custom HTTP header.");
+  writeVerbose(VB_NONE, "                     More headers can be defined by using this option several times.");
   writeVerbose(VB_NONE, "  FORM-DATA:<METHOD>?<FIELDS>");
   writeVerbose(VB_NONE, "                     Methods and fields to send to web service. Valid methods are GET and POST. The actual form");
   writeVerbose(VB_NONE, "                     data to be submitted should also be defined here. Specifically, the fields: username and");
@@ -116,7 +120,7 @@ void showUsage()
   writeVerbose(VB_NONE, "                     Default: \"post?username=&password=\"");
   writeVerbose(VB_NONE, "");
   writeVerbose(VB_NONE, "Usage example: \"-M web-form -m USER-AGENT:\"g3rg3 gerg\" -m FORM:\"webmail/index.php\" -m DENY-SIGNAL:\"deny!\"");
-  writeVerbose(VB_NONE, "                 -m FORM-DATA:\"post?user=&pass=&submit=True\"");
+  writeVerbose(VB_NONE, "                 -m FORM-DATA:\"post?user=&pass=&submit=True\" -m CUSTOM-HEADER:\"Cookie: name=value\"");
 }
 
 // The "main" of the medusa module world - this is what gets called to actually do the work
@@ -203,6 +207,34 @@ int go(sLogin* logins, int argc, char *argv[])
         else
           writeError(ERR_WARNING, "Method USER-AGENT requires value to be set.");
       }
+      else if (strcmp(pOpt, "CUSTOM-HEADER") == 0)
+      {
+        pOpt = strtok_r(NULL, "\0", &strtok_ptr);
+        writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
+
+        if ( pOpt )
+        {
+          if ( psSessionData->nCustomHeaders == 0 )
+          {
+            psSessionData->szCustomHeader = malloc(strlen(pOpt) + 1);
+            memset(psSessionData->szCustomHeader, 0, strlen(pOpt) + 3);
+            strncpy(psSessionData->szCustomHeader, pOpt, strlen(pOpt));
+            strncpy(psSessionData->szCustomHeader + strlen(pOpt), "\r\n", 2);
+            psSessionData->nCustomHeaders = 1;
+          }
+          else
+          {
+            int oldSize = strlen(psSessionData->szCustomHeader);
+            psSessionData->szCustomHeader = realloc(psSessionData->szCustomHeader, oldSize + strlen(pOpt) + 3);
+            memset(psSessionData->szCustomHeader + oldSize, 0, strlen(pOpt) + 3);
+            strncpy(psSessionData->szCustomHeader + oldSize, pOpt, strlen(pOpt));
+            strncpy(psSessionData->szCustomHeader + oldSize + strlen(pOpt), "\r\n", 2);
+            psSessionData->nCustomHeaders += 1;
+          }
+        }
+        else
+          writeError(ERR_WARNING, "Method CUSTOM-HEADER requires value to be set.");
+      }
       else
       {
         writeError(ERR_WARNING, "Invalid method: %s.", pOpt);
@@ -376,6 +408,11 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* _psLogin)
           sprintf(_psSessionData->szDenySignal, "Login Incorrect");
         }
 
+        if (!_psSessionData->szCustomHeader) {
+          _psSessionData->szCustomHeader = malloc(1);
+          memset(_psSessionData->szCustomHeader, 0, 1);
+        }
+
         nState = MSTATE_RUNNING;
         break;
       case MSTATE_RUNNING:
@@ -427,19 +464,36 @@ int initModule(_MODULE_DATA *_psSessionData, sLogin* _psLogin)
   FREE(_psSessionData->szFormRest);
   FREE(_psSessionData->szFormUser);
   FREE(_psSessionData->szFormPass); 
+  FREE(_psSessionData->szCustomHeader); 
   FREE(psCredSet);
   
   return SUCCESS;
 }
 
-int digits(int num)
-{
-  if ( num/10 < 1 ) return 1;
-  else return (1+digits( (int) (num/10) ));
-}
-
 /* Module Specific Functions */
 
+char *urlencodeup(char* szStr){
+  int i=0,j=0;
+  size_t iLen=strlen(szStr);
+  char * szRet = (char*)malloc(sizeof(char)*((iLen*3) + 1));
+
+  static char safechars[] =
+    "abcdefghijklmnopqrstuvwxyz"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+    "0123456789";
+  
+  for(i=0;i<iLen;i++){
+    if(strchr(safechars,szStr[i])){
+      szRet[j++] = szStr[i];
+    }else{
+      sprintf(szRet+j,"%%%02x", (unsigned int)szStr[i]);
+      j+=3;
+    }
+  }
+  szRet[j] = '\0';
+  return szRet;
+}
+
 int sendPost(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char* szPassword)
 {
   char* bufSend = NULL;
@@ -453,7 +507,7 @@ int sendPost(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char* szP
   else
     nFormBufferSize = asprintf(&bufForm, "%s%s&%s%s&%s", _psSessionData->szFormUser, szLogin, _psSessionData->szFormPass, szPassword, _psSessionData->szFormRest); 
 
-  nSendBufferSize = asprintf(&bufSend, "POST /%s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: close\r\nContent-Type: application/x-www-form-urlencoded\r\nContent-Length: %i\r\n\r\n%s", _psSessionData->szDir, _psSessionData->szHostHeader, _psSessionData->szUserAgent, nFormBufferSize, bufForm);
+  nSendBufferSize = asprintf(&bufSend, "POST /%s HTTP/1.0\r\nHost: %s\r\nUser-Agent: %s\r\n%sConnection: close\r\nContent-Type: application/x-www-form-urlencoded\r\nContent-Length: %i\r\n\r\n%s", _psSessionData->szDir, _psSessionData->szHostHeader, _psSessionData->szUserAgent, _psSessionData->szCustomHeader, nFormBufferSize, bufForm);
 
   if (medusaSend(hSocket, bufSend, nSendBufferSize, 0) < 0)
   {
@@ -473,9 +527,9 @@ int sendGet(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char* szPa
   int nRet = SUCCESS;
 
   if ((_psSessionData->szFormRest == NULL) || (_psSessionData->szFormRest[0] == 0))
-    nSendBufferSize = asprintf(&bufSend, "GET /%s?%s%s&%s%s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: close\r\n\r\n", _psSessionData->szDir, _psSessionData->szFormUser, szLogin, _psSessionData->szFormPass, szPassword, _psSessionData->szHostHeader, _psSessionData->szUserAgent);
+    nSendBufferSize = asprintf(&bufSend, "GET /%s?%s%s&%s%s HTTP/1.0\r\nHost: %s\r\nUser-Agent: %s\r\n%sConnection: close\r\n\r\n", _psSessionData->szDir, _psSessionData->szFormUser, szLogin, _psSessionData->szFormPass, szPassword, _psSessionData->szHostHeader, _psSessionData->szUserAgent, _psSessionData->szCustomHeader);
   else
-    nSendBufferSize = asprintf(&bufSend, "GET /%s?%s%s&%s%s&%s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: close\r\n\r\n", _psSessionData->szDir, _psSessionData->szFormUser, szLogin, _psSessionData->szFormPass, szPassword, _psSessionData->szFormRest, _psSessionData->szHostHeader, _psSessionData->szUserAgent);
+    nSendBufferSize = asprintf(&bufSend, "GET /%s?%s%s&%s%s&%s HTTP/1.0\r\nHost: %s\r\nUser-Agent: %s\r\n%sConnection: close\r\n\r\n", _psSessionData->szDir, _psSessionData->szFormUser, szLogin, _psSessionData->szFormPass, szPassword, _psSessionData->szFormRest, _psSessionData->szHostHeader, _psSessionData->szUserAgent, _psSessionData->szCustomHeader);
   
   if (medusaSend(hSocket, bufSend, nSendBufferSize, 0) < 0)
   {
@@ -492,16 +546,19 @@ int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPor
   char* pReceiveBuffer = NULL;
   int nReceiveBufferSize, nRet;
   char* pTemp = NULL;
+  char* szPasswordEncoded = NULL;
+  
+  szPasswordEncoded = urlencodeup(szPassword);
 
   switch(_psSessionData->nFormType)
   {
     case FORM_GET:
       writeError(ERR_DEBUG_MODULE, "[%s] Sending Web Form Authentication (GET).", MODULE_NAME);
-      nRet = sendGet(hSocket, _psSessionData, szLogin, szPassword);
+      nRet = sendGet(hSocket, _psSessionData, szLogin, szPasswordEncoded);
       break;
     case FORM_POST:
       writeError(ERR_DEBUG_MODULE, "[%s] Sending Web Form Authentication (POST).", MODULE_NAME);
-      nRet = sendPost(hSocket, _psSessionData, szLogin, szPassword);
+      nRet = sendPost(hSocket, _psSessionData, szLogin, szPasswordEncoded);
       break;
     default:
       break;
