@@ -35,7 +35,7 @@
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Generic Wrapper Module"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: wrapper.c 1271 2009-11-25 00:24:58Z jmk $"
+#define MODULE_VERSION_SVN "$Id: wrapper.c 1406 2010-09-02 18:29:18Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 
@@ -152,7 +152,9 @@ int go(sLogin* logins, int argc, char *argv[])
         pOpt = strtok_r(NULL, "\0", &strtok_ptr);
         writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
 
-        if (strcmp(pOpt, "SINGLE") == 0)
+        if (pOpt == NULL)
+          writeError(ERR_WARNING, "Method TYPE requires value to be set.");
+        else if (strcmp(pOpt, "SINGLE") == 0)
           psSessionData->nType = TYPE_SINGLE;
         else if (strcmp(pOpt, "STDIN") == 0)
           psSessionData->nType = TYPE_STDIN;
@@ -437,7 +439,7 @@ int initProcess(_MODULE_DATA* _psSessionData, sLogin* psLogin)
         
         setlinebuf(stdout);
 
-        writeError() below does not function since STDOUT is directed threw the pipe.
+        writeError() below does not function since STDOUT is directed through the pipe.
         No real way to tell the user that the child failed...
       */
       
